@@ -6,7 +6,9 @@
 
 @TOP@
 
+#undef  BST_ARCH_NAME
 #undef  BST_VERSION
+#undef  BST_VERSION_HINT
 #undef  BST_DFL_KEYTABLE
 #undef  BST_WITH_XKB
 #undef  BST_WITH_GDK_PIXBUF
