@@ -1,24 +1,25 @@
 /* BEAST - Bedevilled Audio System
- * Copyright (C) 1998, 1999 Olaf Hoehmann and Tim Janik
+ * Copyright (C) 1998, 1999, 2000 Olaf Hoehmann and Tim Janik
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this program; if not, write to the Free Software
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
  */
 #include	"bstapp.h"
 
 #include	"../PKG_config.h"
 
+#include	"bstsampleshell.h"
 #include	"bstsongshell.h"
 #include	"bstsnetshell.h"
 #include	"bstfiledialog.h"
@@ -26,6 +27,7 @@
 #include	"bstheartmonitor.h"
 #include	"bstgconfig.h"
 #include	"bstpreferences.h"
+#include	"bstpatterneditor.h"
 
 
 
@@ -82,6 +84,7 @@ static GtkItemFactoryEntry menubar_entries[] =
   { "/_Help",				NULL,		NULL, 0,			"<LastBranch>" },
   { "/Help/<<<<<<",			NULL,		NULL, 0,			"<Tearoff>" },
   { "/Help/_FAQ...",			NULL,		BST_OP (HELP_FAQ),		"<Item>" },
+  { "/Help/_Keytable...",		NULL,		BST_OP (HELP_KEYTABLE),		"<Item>" },
   { "/Help/_Heart...",			NULL,		BST_OP (HELP_HEART),		"<Item>" },
   { "/Help/Synthesis _Networks...",	NULL,		BST_OP (HELP_NETWORKS),		"<Item>" },
   { "/Help/-----",			NULL,		NULL, 0,			"<Separator>" },
@@ -205,6 +208,26 @@ app_set_title (BstApp *app)
   gtk_window_set_wmclass (window, window->title, g_get_prgname ());
 }
 
+static void
+project_cross_changes (BseContainer *container,
+		       BstApp       *app)
+{
+  BseProject *project = BSE_PROJECT (container);
+  GSList *slist;
+
+ restart:
+  for (slist = project->supers; slist; slist = slist->next)
+    {
+      BseItem *item = BSE_ITEM (slist->data);
+      
+      if (BSE_IS_SAMPLE (item) && BST_SAMPLE_SWEEP && !bse_item_has_cross_owners (item))
+	{
+	  bse_container_remove_item (container, item);
+	  goto restart;
+	}
+    }
+}
+
 static void
 bst_app_destroy (GtkObject *object)
 {
@@ -216,6 +239,12 @@ bst_app_destroy (GtkObject *object)
       bse_object_remove_notifiers_by_func (BSE_OBJECT (app->project),
 					   app_set_title,
 					   app);
+      bse_object_remove_notifiers_by_func (BSE_OBJECT (app->project),
+					   bst_app_reload_supers,
+					   app);
+      bse_object_remove_notifiers_by_func (BSE_OBJECT (app->project),
+					   project_cross_changes,
+					   app);
       bse_object_unref (BSE_OBJECT (app->project));
       app->project = NULL;
     }
@@ -231,14 +260,22 @@ bst_app_destroy (GtkObject *object)
 BstApp*
 bst_app_new (BseProject *project)
 {
+  GdkGeometry geometry;
   GtkWidget *widget;
   BstApp *app;
 
   g_return_val_if_fail (BSE_IS_PROJECT (project), NULL);
 
-  widget = gtk_widget_new (BST_TYPE_APP, NULL);
+  widget = gtk_widget_new (BST_TYPE_APP,
+			   "default_width", 640,
+			   "default_height", 512,
+			   NULL);
   app = BST_APP (widget);
 
+  geometry.min_width = 320;
+  geometry.min_height = 450;
+  gtk_window_set_geometry_hints (GTK_WINDOW (widget), NULL, &geometry, GDK_HINT_MIN_SIZE);
+
   bst_status_bar_ensure (GTK_WINDOW (app));
 
   bse_object_ref (BSE_OBJECT (project));
@@ -247,6 +284,18 @@ bst_app_new (BseProject *project)
 				"name-set",
 				app_set_title,
 				app);
+  bse_object_add_data_notifier (BSE_OBJECT (project),
+				"item-added",
+				bst_app_reload_supers,
+				app);
+  bse_object_add_data_notifier (BSE_OBJECT (project),
+				"item-removed",
+				bst_app_reload_supers,
+				app);
+  bse_object_add_notifier (BSE_OBJECT (project),
+			   "cross-changes",
+			   project_cross_changes,
+			   app);
   app_set_title (app);
 
   bst_app_reload_supers (app);
@@ -281,9 +330,9 @@ bst_app_menu_factory (BstApp *app)
   return gtk_object_get_data (GTK_OBJECT (app), bst_app_factories_path);
 }
 
-static void
-bst_app_add_super (BstApp   *app,
-		   BseSuper *super)
+static GtkWidget*
+bst_app_create_super_shell (BstApp   *app,
+			    BseSuper *super)
 {
   GtkWidget *shell = NULL;
 
@@ -295,36 +344,82 @@ bst_app_add_super (BstApp   *app,
     shell = gtk_widget_new (BST_TYPE_SNET_SHELL,
 			    "visible", TRUE,
 			    NULL);
+  else if (BSE_IS_SAMPLE (super))
+    shell = gtk_widget_new (BST_TYPE_SAMPLE_SHELL,
+			    "visible", TRUE,
+			    NULL);
   else
     {
-      g_message ("FIXME: skipping dialog for %s", BSE_OBJECT_TYPE_NAME (super));
-      return;
+      g_warning ("unknown super type `%s'", BSE_OBJECT_TYPE_NAME (super));
+      return NULL;
     }
 
-  /* gtk_notebook_popup_enable (app->notebook); */
-  gtk_notebook_append_page (app->notebook,
-			    shell,
-			    gtk_widget_new (GTK_TYPE_LABEL,
-					    "visible", TRUE,
-					    NULL));
   bst_super_shell_set_super (BST_SUPER_SHELL (shell), super);
+
+  gtk_widget_ref (shell);
+  gtk_object_sink (GTK_OBJECT (shell));
+
+  return shell;
 }
 
 void
 bst_app_reload_supers (BstApp *app)
 {
+  GtkWidget *old_page;
+  GSList *page_list = NULL;
   GSList *slist;
 
   g_return_if_fail (BST_IS_APP (app));
 
-  gtk_widget_hide (GTK_WIDGET (app->notebook));
+  // gtk_widget_hide (GTK_WIDGET (app->notebook));
 
-  gtk_container_foreach (GTK_CONTAINER (app->notebook), (GtkCallback) gtk_widget_destroy, NULL);
+  old_page = app->notebook->cur_page ? app->notebook->cur_page->child : NULL;
+  while (app->notebook->cur_page && app->notebook->cur_page->child)
+    {
+      gtk_object_ref (GTK_OBJECT (app->notebook->cur_page->child));
+      page_list = g_slist_prepend (page_list, app->notebook->cur_page->child);
+      gtk_container_remove (GTK_CONTAINER (app->notebook), page_list->data);
+    }
+
+  // gtk_container_foreach (GTK_CONTAINER (app->notebook), (GtkCallback) gtk_widget_destroy, NULL);
 
   for (slist = app->project->supers; slist; slist = slist->next)
-    bst_app_add_super (app, slist->data);
+    {
+      GtkWidget *label, *page = NULL;
+      GSList *node;
 
-  gtk_widget_show (GTK_WIDGET (app->notebook));
+      for (node = page_list; node; node = node->next)
+	if (BST_SUPER_SHELL (node->data)->super == slist->data)
+	  {
+	    page = node->data;
+	    page_list = g_slist_remove (page_list, page);
+	    break;
+	  }
+      if (!page)
+	page = bst_app_create_super_shell (app, slist->data);
+      label = gtk_widget_new (GTK_TYPE_LABEL,
+			      "visible", TRUE,
+			      "width", BST_TAB_WIDTH ? BST_TAB_WIDTH : -1,
+			      NULL);
+      gtk_notebook_append_page (app->notebook,
+				page,
+				label);
+      gtk_notebook_set_tab_label_packing (app->notebook, page, FALSE, TRUE, GTK_PACK_START);
+      bst_super_shell_update_parent (BST_SUPER_SHELL (page));
+      gtk_widget_unref (page);
+    }
+  if (old_page && old_page->parent == GTK_WIDGET (app->notebook))
+    gtk_notebook_set_page (app->notebook,
+			   gtk_notebook_page_num (app->notebook,
+						  old_page));
+  for (slist = page_list; slist; slist = slist->next)
+    {
+      gtk_widget_destroy (slist->data);
+      gtk_widget_unref (slist->data);
+    }
+  g_slist_free (page_list);
+
+  // gtk_widget_show (GTK_WIDGET (app->notebook));
 }
 
 static gboolean
@@ -390,7 +485,8 @@ bst_app_operate (BstApp *app,
   static GtkWidget *bst_dialog_save = NULL;
   static GtkWidget *bst_preferences = NULL;
   GtkWidget *widget, *shell;
-  gchar *help_file;
+  gchar *help_file = NULL, *help_title = NULL;
+  GString *help_string = NULL;
 
   g_return_if_fail (BST_IS_APP (app));
   g_return_if_fail (bst_app_can_operate (app, op));
@@ -400,6 +496,8 @@ bst_app_operate (BstApp *app,
 
   gtk_widget_ref (widget);
 
+  bst_status_window_push (widget);
+
   switch (op)
     {
       BstSuperShell *super_shell;
@@ -411,12 +509,13 @@ bst_app_operate (BstApp *app,
       if (1)
 	{
 	  BseProject *project = bse_project_new ("Untitled.bse");
-	  BstApp *napp = bst_app_new (project);
+	  BstApp *new_app;
 
+	  new_app = bst_app_new (project);
 	  bse_object_unref (BSE_OBJECT (project));
-	  if (0)
-	    bst_app_operate (napp, BST_OP_PROJECT_NEW_SONG);
-	  gtk_idle_show_widget (GTK_WIDGET (napp));
+	  if (0) /* FIXME: config values */
+	    bst_app_operate (new_app, BST_OP_PROJECT_NEW_SONG);
+	  gtk_idle_show_widget (GTK_WIDGET (new_app));
 	}
       break;
     case BST_OP_PROJECT_OPEN:
@@ -455,22 +554,28 @@ bst_app_operate (BstApp *app,
       break;
     case BST_OP_PROJECT_NEW_SONG:
       song = bse_song_new (BST_APP_PROJECT (app), BSE_DFL_SONG_N_CHANNELS);
-      bst_app_reload_supers (app);
       super_shell = bst_super_shell_from_super (BSE_SUPER (song));
       bst_super_shell_operate (super_shell, BST_OP_PATTERN_ADD);
       bse_object_unref (BSE_OBJECT (song));
       break;
     case BST_OP_PROJECT_NEW_SNET:
       snet = bse_snet_new (BST_APP_PROJECT (app), NULL);
-      bst_app_reload_supers (app);
       super_shell = bst_super_shell_from_super (BSE_SUPER (snet));
       bse_object_unref (BSE_OBJECT (snet));
       break;
     case BST_OP_PROJECT_PLAY:
+      if (BSE_SOURCE_PREPARED (app->project))
+	{
+	  bse_project_stop_playback (app->project);
+	  bst_status_set (0, "Restarting Playback", NULL);
+	}
+      else
+	bst_status_set (0, "Starting Playback", NULL);
       bse_project_start_playback (app->project);
       break;
     case BST_OP_PROJECT_STOP:
       bse_project_stop_playback (app->project);
+      bst_status_set (0, "Stopping Playback", NULL);
       break;
     case BST_OP_DIALOG_PREFERENCES:
       if (!bst_preferences)
@@ -483,8 +588,11 @@ bst_app_operate (BstApp *app,
 	  bse_object_unref (BSE_OBJECT (gconf));
 	  gtk_widget_show (bst_preferences);
 	  deflt = BST_PREFERENCES (bst_preferences)->close; // apply;
-	  bst_preferences = bst_subwindow_new (NULL, &bst_preferences, bst_preferences, 0);
-	  gtk_window_set_title (GTK_WINDOW (bst_preferences), "BEAST: Preferences");
+	  bst_preferences = bst_adialog_new (NULL,
+					     &bst_preferences, bst_preferences,
+					     0,
+					     "title", "BEAST: Preferences",
+					     NULL);
 	  gtk_widget_grab_default (deflt);
 	}
       gtk_widget_showraise (bst_preferences);
@@ -502,8 +610,10 @@ bst_app_operate (BstApp *app,
 	    {
 	      hmon = bst_heart_monitor_new (heart);
 	      gtk_widget_show (hmon);
-	      hmon = bst_subwindow_new (NULL, NULL, hmon, 0);
-	      gtk_window_set_title (GTK_WINDOW (hmon), "BEAST: Device Monitor");
+	      hmon = bst_adialog_new (NULL, NULL, hmon,
+				      0,
+				      "title", "BEAST: Device Monitor",
+				      NULL);
 	    }
 	  gtk_widget_showraise (hmon);
 	}
@@ -521,13 +631,20 @@ bst_app_operate (BstApp *app,
       gtk_widget_queue_draw (GTK_WIDGET (app->notebook));
       break;
     case BST_OP_HELP_FAQ:
-      help_file = "faq.txt";
+      help_file = g_strconcat (BST_PATH_DOCS, "/faq.txt", NULL);
+      help_title = help_file;
+      goto case_help_dialog;
+    case BST_OP_HELP_KEYTABLE:
+      help_string = bst_pattern_editor_class_keydump (gtk_type_class (BST_TYPE_PATTERN_EDITOR));
+      help_title = "Keytable";
       goto case_help_dialog;
     case BST_OP_HELP_HEART:
-      help_file = "bse-heart.txt";
+      help_file = g_strconcat (BST_PATH_DOCS, "/bse-heart.txt", NULL);
+      help_title = help_file;
       goto case_help_dialog;
     case BST_OP_HELP_NETWORKS:
-      help_file = "bse-networks.txt";
+      help_file = g_strconcat (BST_PATH_DOCS, "/bse-networks.txt", NULL);
+      help_title = help_file;
       goto case_help_dialog;
     case BST_OP_HELP_ABOUT:
       break;
@@ -536,16 +653,21 @@ bst_app_operate (BstApp *app,
 	{
 	  gchar *string;
 
-	  string = g_strconcat (BST_PATH_DOCS, "/", help_file, NULL);
-	  bst_help_dialogs[op - BST_OP_HELP_FIRST] = bst_subwindow_new (NULL,
-									&bst_help_dialogs[op - BST_OP_HELP_FIRST],
-									bst_text_view_from_file (string),
-									0);
-	  g_free (string);
-	  string = g_strconcat ("BEAST: ", help_file, NULL);
-	  gtk_window_set_title (GTK_WINDOW (bst_help_dialogs[op - BST_OP_HELP_FIRST]), string);
+	  string = g_strconcat ("BEAST: ", help_title, NULL);
+	  bst_help_dialogs[op - BST_OP_HELP_FIRST] = bst_adialog_new (NULL,
+								      &bst_help_dialogs[op - BST_OP_HELP_FIRST],
+								      bst_text_view_from (help_string,
+											  help_file,
+											  "-misc-fixed-*-*-*-*-15-*-*-*-*-*-*-*",
+											  "-misc-fixed-*-*-*-*-*-*-*-*-*-*-*-*"),
+								      BST_ADIALOG_DESTROY_ON_HIDE,
+								      "title", string,
+								      NULL);
 	  g_free (string);
 	}
+      g_free (help_file);
+      if (help_string)
+	g_string_free (help_string, TRUE);
       gtk_widget_showraise (bst_help_dialogs[op - BST_OP_HELP_FIRST]);
       break;
     default:
@@ -554,6 +676,8 @@ bst_app_operate (BstApp *app,
       break;
     }
 
+  bst_status_window_pop ();
+
   bst_update_can_operate (widget);
 
   gtk_widget_unref (widget);
@@ -628,6 +752,7 @@ bst_app_can_operate (BstApp *app,
       return TRUE;
       // case BST_OP_HELP_ABOUT:
     case BST_OP_HELP_FAQ:
+    case BST_OP_HELP_KEYTABLE:
     case BST_OP_HELP_NETWORKS:
     case BST_OP_HELP_HEART:
       return TRUE;
