@@ -1,24 +1,24 @@
-/* BST - The GTK+ Layout Engine
- * Copyright (C) 1998, 1999 Tim Janik
+/* BEAST - Bedevilled Audio System
+ * Copyright (C) 1998, 1999, 2000 Olaf Hoehmann and Tim Janik
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
- * Library General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the Free Software
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
  */
 #include	"bstfiledialog.h"
 
-#include	<stdio.h>
-#include	<fcntl.h>
+#include	"bststatusbar.h"
+#include	"bstmenus.h"
 #include	<errno.h>
 
 
@@ -68,7 +68,8 @@ static void
 bst_file_dialog_init (BstFileDialog *fd)
 {
   gtk_widget_set (GTK_WIDGET (fd),
-		  "GtkWindow::title", "BEAST FileDialog",
+		  "title", "BEAST FileDialog",
+		  "window_position", GTK_WIN_POS_MOUSE,
 		  NULL);
   
   gtk_file_selection_heal (GTK_FILE_SELECTION (fd));
@@ -94,9 +95,12 @@ bst_file_dialog_open (BstFileDialog *fd)
   error = bse_storage_input_file (storage, file_name);
 
   if (error)
-    g_message ("failed to open `%s': %s", /* FIXME */
-	       file_name,
-	       bse_error_blurb (error));
+    {
+      g_message ("failed to open `%s': %s", /* FIXME */
+		 file_name,
+		 bse_error_blurb (error));
+      bst_status_printf (0, bse_error_blurb (error), "Failed to open `%s'", file_name);
+    }
   else
     {
       BseProject *project = bse_project_new (file_name);
@@ -109,7 +113,13 @@ bst_file_dialog_open (BstFileDialog *fd)
 		   file_name,
 		   bse_error_blurb (error));
       app = bst_app_new (project);
+      bst_status_window_push (app);
       bse_object_unref (BSE_OBJECT (project));
+      bst_status_printf (error ? 0 : 100,
+			 error ? "Failed" : "Done",
+			 "Loading project `%s'",
+			 file_name);
+      bst_status_window_pop ();
       gtk_idle_show_widget (GTK_WIDGET (app));
     }
   
@@ -135,20 +145,53 @@ bst_file_dialog_save (BstFileDialog *fd)
   if (radio && GTK_TOGGLE_BUTTON (radio)->active)
     ;
 
+  bst_status_window_push (app);
+
+ retry_saving:
+
   error = bse_project_store_bse (app->project, file_name);
-  
-  if (error)
+
+  /* offer retry if file exists
+   */
+  if (error == BSE_ERROR_FILE_EXISTS)
     {
-      g_message ("failed to save `%s' to `%s': %s", /* FIXME */
-		 BSE_OBJECT_NAME (app->project),
-		 file_name,
-		 bse_error_blurb (error));
-      g_free (file_name);
-      return;
+      GtkWidget *choice;
+      gchar *title = g_strdup_printf ("Saving project `%s'", BSE_OBJECT_NAME (app->project));
+      gchar *text = g_strdup_printf ("Failed to save\n`%s'\nto\n`%s':\n%s",
+				     BSE_OBJECT_NAME (app->project),
+				     file_name,
+				     bse_error_blurb (error));
+      
+      choice = bst_choice_dialog_createv (BST_CHOICE_TITLE (title),
+					  BST_CHOICE_TEXT (text),
+					  BST_CHOICE_D (1, "Overwrite", NONE),
+					  BST_CHOICE (0, "Cancel", NONE),
+					  BST_CHOICE_END);
+      g_free (title);
+      g_free (text);
+      if (bst_choice_modal (choice, 0, 0) == 1)
+	{
+	  bst_choice_destroy (choice);
+	  if (unlink (file_name) < 0)
+	    bst_status_printf (0, g_strerror (errno), "Deleting `%s'", file_name);
+	  else
+	    goto retry_saving;
+	}
+      else
+	bst_choice_destroy (choice);
+    }
+  else
+    {
+      bst_status_printf (error ? 0 : 100,
+			 error ? bse_error_blurb (error) : "Done",
+			 "Saving project `%s'",
+			 file_name);
+      if (!error)
+	gtk_widget_destroy (GTK_WIDGET (fd));
     }
 
+  bst_status_window_pop ();
   g_free (file_name);
-  gtk_widget_destroy (GTK_WIDGET (fd));
 }
 
 GtkWidget*
@@ -240,63 +283,3 @@ bst_file_dialog_new_save (BstApp *app)
 
   return dialog;
 }
-
-GtkWidget*
-bst_text_view_from_file (const gchar *file_name) /* FIXME: should go into misc.c or utils.c */
-{
-  gint fd;
-  GtkWidget *hbox, *text, *sb;
-  
-  g_return_val_if_fail (file_name != NULL, NULL);
-  
-  hbox = gtk_widget_new (GTK_TYPE_HBOX,
-			 "visible", TRUE,
-			 "homogeneous", FALSE,
-			 "spacing", 0,
-			 "border_width", 5,
-			 NULL);
-  sb = gtk_vscrollbar_new (NULL);
-  gtk_widget_show (sb);
-  gtk_box_pack_end (GTK_BOX (hbox), sb, FALSE, TRUE, 0);
-  text = gtk_widget_new (GTK_TYPE_TEXT,
-			 "visible", TRUE,
-			 "vadjustment", GTK_RANGE (sb)->adjustment,
-			 "editable", FALSE,
-			 "word_wrap", TRUE,
-			 "line_wrap", TRUE,
-			 "width", 500,
-			 "height", 500,
-			 "parent", hbox,
-			 NULL);
-  
-  fd = open (file_name, O_RDONLY, 0);
-  if (fd >= 0)
-    {
-      gchar buffer[512];
-      guint n;
-      
-      do
-	{
-	  do
-	    n = read (fd, buffer, 512);
-	  while (n < 0 && errno == EINTR); /* don't mind signals */
-	  
-	  gtk_text_insert (GTK_TEXT (text), NULL, NULL, NULL, buffer, n);
-	}
-      while (n > 0);
-      close (fd);
-
-      if (n < 0)
-	fd = -1;
-    }
-  if (fd < 0)
-    {
-      gchar *error;
-
-      error = g_strconcat ("Failed to load \"", file_name, "\":\n", g_strerror (errno), NULL);
-      gtk_text_insert (GTK_TEXT (text), NULL, NULL, NULL, error, strlen (error));
-      g_free (error);
-    }
-
-  return hbox;
-}
