@@ -1,28 +1,26 @@
 /* BEAST - Bedevilled Audio System
- * Copyright (C) 1998, 1999 Olaf Hoehmann and Tim Janik
+ * Copyright (C) 1998, 1999, 2000 Olaf Hoehmann and Tim Janik
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this program; if not, write to the Free Software
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
  */
 #include        "bstdefs.h"
 
 #include        "bstapp.h"
 #include        "bstsamplerepo.h"
-#include        "bstprocedure.h"
 #include	"bstxkb.h"
 #include	"bstkeytables.h"
-#include	"bstmenus.h"
 #include	"bstgconfig.h"
 #include	"bstpreferences.h"
 #include	"../PKG_config.h"
@@ -39,16 +37,19 @@
 static void			bst_parse_args		(gint        *argc_p,
 							 gchar     ***argv_p);
 static BstKeyTablePatch*	bst_key_table_from_xkb	(const gchar *display);
+static void			bst_print_blurb		(FILE	     *fout,
+							 gboolean     print_help);
 
 
 /* --- variables --- */
 static guint        args_changed_signal_id = 0;
 BstDebugFlags       bst_debug_flags = 0;
-static GDebugKey       bst_debug_keys[] = { /* keep in sync with bstdefs.h */
+static GDebugKey    bst_debug_keys[] = { /* keep in sync with bstdefs.h */
   { "keytable",		BST_DEBUG_KEYTABLE, },
   { "samples",		BST_DEBUG_SAMPLES, },
 };
-static const guint bst_n_debug_keys = sizeof (bst_debug_keys) / sizeof (bst_debug_keys[0]);
+static const guint  bst_n_debug_keys = sizeof (bst_debug_keys) / sizeof (bst_debug_keys[0]);
+static gboolean     arg_force_xkb = FALSE;
 static const gchar *bst_rc_string =
 ( "style'BstTooltips-style'"
   "{"
@@ -67,8 +68,6 @@ main (int   argc,
   BstApp *app = NULL;
   guint i;
   
-  g_message ("BEAST: pid = %u", getpid ());
-  
   /* initialize BSE and preferences
    */
   bse_init (&argc, &argv);
@@ -76,13 +75,16 @@ main (int   argc,
   
   /* initialize GUI libraries and pre-parse args
    */
+  bst_parse_args (&argc, &argv);
+
+  g_message ("BEAST: pid = %u", getpid ());
+  
   gtk_init (&argc, &argv);
   gtk_rc_parse_string (bst_rc_string);
   gdk_rgb_init ();
   gnome_type_init ();
   // gnome_init (PROGRAM, VERSION, argc, argv);
   bst_free_radio_button_get_type ();
-  bst_parse_args (&argc, &argv);
   
   /* parse rc file */
   if (1)
@@ -90,7 +92,7 @@ main (int   argc,
       BseGConfig *gconf;
       BseErrorType error;
       gchar *file_name;
-
+      
       file_name = BST_STRDUP_RC_FILE ();
       gconf = bse_object_new (BST_TYPE_GCONFIG, NULL);
       bse_gconfig_revert (gconf);
@@ -104,12 +106,12 @@ main (int   argc,
       bse_object_unref (BSE_OBJECT (gconf));
       g_free (file_name);
     }
-
+  
   /* setup default keytable for pattern editor class
    */
   bst_key_table_install_patch (bst_key_table_from_xkb (gdk_get_display ()));
-
-
+  
+  
   /* check load BSE plugins to register types
    */
   if (1)
@@ -148,7 +150,7 @@ main (int   argc,
   if (!pdev && BSE_TYPE_ID (BsePcmDeviceAlsa) && !BST_DISABLE_ALSA)
     {
       BseErrorType error;
-
+      
       pdev = (BsePcmDevice*) bse_object_new (BSE_TYPE_ID (BsePcmDeviceAlsa), NULL); /* FIXME: TYPE_ID */
       error = bse_pcm_device_update_caps (pdev);
       if (error && error != BSE_ERROR_DEVICE_BUSY)
@@ -160,7 +162,7 @@ main (int   argc,
   if (!pdev && BSE_TYPE_ID (BsePcmDeviceOSS))
     {
       BseErrorType error;
-
+      
       pdev = (BsePcmDevice*) bse_object_new (BSE_TYPE_ID (BsePcmDeviceOSS), NULL); /* FIXME: TYPE_ID */
       error = bse_pcm_device_update_caps (pdev);
       if (error && error != BSE_ERROR_DEVICE_BUSY)
@@ -175,8 +177,8 @@ main (int   argc,
   bse_object_unref (BSE_OBJECT (pdev));
   bse_heart_set_default_odevice ("Master");
   bse_heart_set_default_idevice ("Master");
-
-
+  
+  
   /* register sample repositories
    */
   bst_sample_repo_init ();
@@ -228,23 +230,23 @@ main (int   argc,
   /* and away into the main loop
    */
   gtk_main ();
-
+  
   /* stop everything playing */
   bse_heart_reset_all_attach ();
-
+  
   /* FXME: wrt cleanup cycles */
-
+  
   /* perform necessary cleanup cycles */
   while (g_main_iteration (FALSE))
     ;
-
+  
   /* save rc file */
   if (1)
     {
       BseGConfig *gconf;
       BseErrorType error;
       gchar *file_name;
-
+      
       gconf = bse_object_new (BST_TYPE_GCONFIG, NULL);
       bse_gconfig_revert (gconf);
       file_name = BST_STRDUP_RC_FILE ();
@@ -254,10 +256,10 @@ main (int   argc,
 	g_warning ("error saving rc-file \"%s\": %s", file_name, bse_error_blurb (error));
       g_free (file_name);
     }
-
+  
   /* remove pcm devices */
   bse_heart_unregister_all_devices ();
-
+  
   /* perform necessary cleanup cycles */
   while (g_main_iteration (FALSE))
     ;
@@ -357,6 +359,25 @@ bst_parse_args (int    *argc_p,
 	    }
 	  argv[i] = NULL;
 	}
+      else if (strcmp ("--force-xkb", argv[i]) == 0)
+	{
+	  arg_force_xkb = TRUE;
+          argv[i] = NULL;
+	}
+      else if (strcmp ("-h", argv[i]) == 0 ||
+	       strcmp ("--help", argv[i]) == 0)
+	{
+	  bst_print_blurb (stderr, TRUE);
+          argv[i] = NULL;
+	  exit (0);
+	}
+      else if (strcmp ("-v", argv[i]) == 0 ||
+	       strcmp ("--version", argv[i]) == 0)
+	{
+	  bst_print_blurb (stderr, FALSE);
+	  argv[i] = NULL;
+	  exit (0);
+	}
     }
   
   e = 0;
@@ -377,6 +398,58 @@ bst_parse_args (int    *argc_p,
     *argc_p = e;
 }
 
+static void
+bst_print_blurb (FILE    *fout,
+		 gboolean print_help)
+{
+  if (!print_help)
+    {
+      fprintf (fout, "BEAST version %s (%s)\n", BST_VERSION, BST_VERSION_HINT);
+      fprintf (fout, "Libraries: ");
+      fprintf (fout, "BSE %u.%u.%u", bse_major_version, bse_minor_version, bse_micro_version);
+      fprintf (fout, ", GTK+ %u.%u.%u", gtk_major_version, gtk_minor_version, gtk_micro_version);
+      fprintf (fout, ", GLib %u.%u.%u", glib_major_version, glib_minor_version, glib_micro_version);
+#ifdef BST_WITH_GDK_PIXBUF
+      fprintf (fout, ", GdkPixbuf");
+#endif
+#ifdef BST_WITH_XKB
+      fprintf (fout, ", XKBlib");
+#endif
+      fprintf (fout, "\n");
+      fprintf (fout, "Compiled for: %s\n", BST_ARCH_NAME);
+      fprintf (fout, "\n");
+      fprintf (fout, "Doc-path:    %s\n", BST_PATH_DOCS);
+      fprintf (fout, "Plugin-path: %s\n", BSE_PATH_PLUGINS);
+      fprintf (fout, "Sample-path: %s\n", BST_PATH_DATA_SAMPLES);
+      fprintf (fout, "\n");
+      fprintf (fout, "BEAST comes with ABSOLUTELY NO WARRANTY.\n");
+      fprintf (fout, "You may redistribute copies of BEAST under the terms of\n");
+      fprintf (fout, "the GNU General Public License which can be found in the\n");
+      fprintf (fout, "BEAST source package. Sources, examples and contact\n");
+      fprintf (fout, "information are available at http://beast.gtk.org\n");
+    }
+  else
+    {
+      fprintf (fout, "Usage: beast [options] [files...]\n");
+      fprintf (fout, "  --beast-debug=keys		enable certain BEAST debug stages\n");
+      fprintf (fout, "  --beast-no-debug=keys		disable certain BEAST debug stages\n");
+      fprintf (fout, "  --force-xkb			force XKB keytable queries\n");
+      fprintf (fout, "  --bse-debug=keys		enable certain BSE debug stages\n");
+      fprintf (fout, "  --bse-no-debug=keys		disable certain BSE debug stages\n");
+      fprintf (fout, "  -h, --help			show this help message\n");
+      fprintf (fout, "  -v, --version			print version informations\n");
+      fprintf (fout, "  --display=DISPLAY		X server to contact; see X(1)\n");
+      fprintf (fout, "  --no-xshm			disable use of X shared memory extension\n");
+      fprintf (fout, "  --gtk-debug=FLAGS		Gtk+ debugging flags to enable\n");
+      fprintf (fout, "  --gtk-no-debug=FLAGS		Gtk+ debugging flags to disable\n");
+      fprintf (fout, "  --gtk-module=MODULE		load additional Gtk+ modules\n");
+      fprintf (fout, "  --gdk-debug=FLAGS		Gdk debugging flags to enable\n");
+      fprintf (fout, "  --gdk-no-debug=FLAGS		Gdk debugging flags to disable\n");
+      fprintf (fout, "  --g-fatal-warnings		make warnings fatal (abort)\n");
+      fprintf (fout, "  --sync   			do all X calls synchronously\n");
+    }
+}
+
 static BstKeyTablePatch*
 bst_key_table_from_xkb_symbol (const gchar *xkb_symbol)
 {
@@ -455,23 +528,23 @@ static BstKeyTablePatch*
 bst_key_table_from_xkb (const gchar *display)
 {
   BstKeyTablePatch *patch = NULL;
-
-  if (!BST_XKB_FORCE_QUERY && BST_XKB_SYMBOL)
+  
+  if (!BST_XKB_FORCE_QUERY && !arg_force_xkb && BST_XKB_SYMBOL)
     patch = bst_key_table_patch_find (BST_XKB_SYMBOL);
-
-  if (!patch && !BST_XKB_FORCE_QUERY && BST_XKB_SYMBOL)
+  
+  if (!patch && !BST_XKB_FORCE_QUERY && !arg_force_xkb && BST_XKB_SYMBOL)
     {
       BST_IF_DEBUG (KEYTABLE)
 	g_message ("Failed to find keytable \"%s\"", BST_XKB_SYMBOL);
     }
-
+  
   if (!patch)
     {
       gchar *xkb_symbol = NULL;
-
+      
       BST_IF_DEBUG (KEYTABLE)
 	g_message ("Querying keytable layout from X-Server...");
-
+      
       if (bst_xkb_open (display, TRUE))
 	{
 	  xkb_symbol = g_strdup (bst_xkb_get_symbol (TRUE));
@@ -479,11 +552,11 @@ bst_key_table_from_xkb (const gchar *display)
 	    xkb_symbol = g_strdup (bst_xkb_get_symbol (FALSE));
 	  bst_xkb_close ();
 	}
-
+      
       patch = bst_key_table_from_xkb_symbol (xkb_symbol);
       g_free (xkb_symbol);
     }
-
+  
   if (patch)
     {
       BST_IF_DEBUG (KEYTABLE)
@@ -492,78 +565,17 @@ bst_key_table_from_xkb (const gchar *display)
   else
     {
       gchar *name = BST_DFL_KEYTABLE;
-
+      
       BST_IF_DEBUG (KEYTABLE)
 	g_message ("Guessing keytable failed, reverting to \"%s\"", name);
       patch = bst_key_table_patch_find (name);
     }
-
+  
   bst_globals_set_xkb_symbol (patch->identifier);
   
   return patch;
 }
 
-BseIcon*
-bst_icon_from_stock (BstIconId _id) /* static icons, no reference counting needed */
-{
-#include "./icons/noicon.c"
-#include "./icons/mouse_tool.c"
-#include "./icons/properties.c"
-#include "./icons/trash.c"
-#include "./icons/close.c"
-#include "./icons/no_ilink.c"
-#include "./icons/no_olink.c"
-  static const BsePixdata pixdatas[] = {
-    /* BST_ICON_NONE */
-    { 0, 0, 0, NULL, },
-    /* BST_ICON_NOICON */
-    { NOICON_PIXDATA_BYTES_PER_PIXEL | BSE_PIXDATA_1BYTE_RLE,
-      NOICON_PIXDATA_WIDTH, NOICON_PIXDATA_HEIGHT,
-      NOICON_PIXDATA_RLE_PIXEL_DATA, },
-    /* BST_ICON_MOUSE_TOOL */
-    { MOUSE_TOOL_IMAGE_BYTES_PER_PIXEL | BSE_PIXDATA_1BYTE_RLE,
-      MOUSE_TOOL_IMAGE_WIDTH, MOUSE_TOOL_IMAGE_HEIGHT,
-      MOUSE_TOOL_IMAGE_RLE_PIXEL_DATA, },
-    /* BST_ICON_PROPERTIES */
-    { PROPERTIES_IMAGE_BYTES_PER_PIXEL | BSE_PIXDATA_1BYTE_RLE,
-      PROPERTIES_IMAGE_WIDTH, PROPERTIES_IMAGE_HEIGHT,
-      PROPERTIES_IMAGE_RLE_PIXEL_DATA, },
-    /* BST_ICON_TRASH */
-    { TRASH_IMAGE_BYTES_PER_PIXEL | BSE_PIXDATA_1BYTE_RLE,
-      TRASH_IMAGE_WIDTH, TRASH_IMAGE_HEIGHT,
-      TRASH_IMAGE_RLE_PIXEL_DATA, },
-    /* BST_ICON_CLOSE */
-    { CLOSE_IMAGE_BYTES_PER_PIXEL | BSE_PIXDATA_1BYTE_RLE,
-      CLOSE_IMAGE_WIDTH, CLOSE_IMAGE_HEIGHT,
-      CLOSE_IMAGE_RLE_PIXEL_DATA, },
-    /* BST_ICON_NO_ILINK */
-    { NO_ILINK_IMAGE_BYTES_PER_PIXEL | BSE_PIXDATA_1BYTE_RLE,
-      NO_ILINK_IMAGE_WIDTH, NO_ILINK_IMAGE_HEIGHT,
-      NO_ILINK_IMAGE_RLE_PIXEL_DATA, },
-    /* BST_ICON_NO_OLINK */
-    { NO_OLINK_IMAGE_BYTES_PER_PIXEL | BSE_PIXDATA_1BYTE_RLE,
-      NO_OLINK_IMAGE_WIDTH, NO_OLINK_IMAGE_HEIGHT,
-      NO_OLINK_IMAGE_RLE_PIXEL_DATA, },
-  };
-  static const guint n_stock_icons = sizeof (pixdatas) / sizeof (pixdatas[0]);
-  static BseIcon *icons[sizeof (pixdatas) / sizeof (pixdatas[0])] = { NULL, };
-  guint icon_id = _id;
-  
-  g_assert (n_stock_icons == BST_ICON_LAST);
-  g_return_val_if_fail (icon_id < n_stock_icons, NULL);
-  
-  if (!icons[icon_id])
-    {
-      if (!pixdatas[icon_id].encoded_pix_data)
-	return NULL;
-
-      icons[icon_id] = bse_icon_from_pixdata (pixdatas + icon_id); /* static reference */
-      bse_icon_static_ref (icons[icon_id]);
-    }
-
-  return icons[icon_id];
-}
-
 /* read bstdefs.h on this */
 void
 bst_update_can_operate (GtkWidget *widget)
@@ -632,263 +644,3 @@ bst_object_set (gpointer     object,
   
   gtk_object_unref (object);
 }
-
-static gint
-subwindow_button_press_event (GtkWidget      *window,
-			      GdkEventButton *event)
-{
-  gboolean handled = FALSE;
-
-  if (event->button == 3 && event->window == window->window)
-    {
-      handled = TRUE;
-      if (bst_choice_modal (gtk_object_get_data (GTK_OBJECT (window), "subwindow-choice"),
-			    event->button,
-			    event->time) == 1)
-	gtk_widget_hide (window);
-    }
-
-  return handled;
-}
-
-static gint
-subwindow_delete_event (GtkWidget *window)
-{
-  gtk_widget_hide (window);
-
-  return TRUE;
-}
-
-static GtkWidget *subwindow_choice = NULL;
-
-GtkWidget*
-bst_subwindow_new (GtkObject  *alive_host,
-		   GtkWidget **ssubwindow_p,
-		   GtkWidget  *child,
-		   guint       zero)
-{
-  GtkWidget *window;
-
-  g_return_val_if_fail (GTK_IS_WIDGET (child), NULL);
-  g_return_val_if_fail (child->parent == NULL, NULL);
-  g_return_val_if_fail (zero == 0, NULL);
-  if (alive_host)
-    g_return_val_if_fail (GTK_IS_OBJECT (alive_host), NULL);
-  if (ssubwindow_p)
-    g_return_val_if_fail (ssubwindow_p != NULL, NULL);
-
-  if (!subwindow_choice)
-    {
-      subwindow_choice = bst_choice_createv (BST_CHOICE (1, "Close", CLOSE),
-					     BST_CHOICE_END);
-      gtk_widget_set (subwindow_choice,
-		      "object_signal::destroy", bse_nullify_pointer, &subwindow_choice,
-		      NULL);
-    }
-  else
-    gtk_widget_ref (subwindow_choice);
-
-  window = gtk_widget_new (GTK_TYPE_WINDOW,
-			   "auto_shrink", FALSE,
-			   "allow_shrink", FALSE,
-			   "allow_grow", TRUE,
-			   "signal::delete_event", subwindow_delete_event, NULL,
-			   "signal::button_press_event", subwindow_button_press_event, NULL,
-			   "events", GDK_BUTTON_PRESS_MASK,
-			   "child", child,
-			   ssubwindow_p ? "object_signal::destroy" : NULL, bse_nullify_pointer, ssubwindow_p,
-			   NULL);
-  gtk_widget_ref (child);
-  gtk_object_set_data_full (GTK_OBJECT (window), "subwindow-child", child, (GDestroyNotify) gtk_widget_unref);
-  gtk_object_set_data_full (GTK_OBJECT (window), "subwindow-choice", subwindow_choice, (GDestroyNotify) gtk_widget_unref);
-  if (alive_host)
-    gtk_signal_connect_object_while_alive (alive_host,
-					   "destroy",
-					   GTK_SIGNAL_FUNC (gtk_widget_destroy),
-					   GTK_OBJECT (window));
-  else
-    gtk_quit_add_destroy (1, GTK_OBJECT (window));
-
-  return window;
-}
-
-GtkWidget*
-bst_subwindow_get_child (GtkWidget *subwindow)
-{
-  g_return_val_if_fail (GTK_IS_WINDOW (subwindow), NULL);
-
-  return gtk_object_get_data (GTK_OBJECT (subwindow), "subwindow-child");
-}
-
-void
-gtk_widget_showraise (GtkWidget *widget)
-{
-  g_return_if_fail (GTK_IS_WIDGET (widget));
-
-  gtk_widget_show (widget);
-  if (GTK_WIDGET_REALIZED (widget))
-    gdk_window_raise (widget->window);
-}
-
-void
-gtk_toplevel_hide (GtkWidget *widget)
-{
-  g_return_if_fail (GTK_IS_WIDGET (widget));
-
-  widget = gtk_widget_get_toplevel (widget);
-  gtk_widget_hide (widget);
-}
-
-GnomeCanvasItem*
-gnome_canvas_typed_item_at (GnomeCanvas *canvas,
-			    GtkType      item_type,
-			    gdouble      world_x,
-			    gdouble      world_y)
-{
-  GnomeCanvasItem *item;
-
-  g_return_val_if_fail (GNOME_IS_CANVAS (canvas), NULL);
-
-  item = gnome_canvas_get_item_at (canvas, world_x, world_y);
-  while (item && !gtk_type_is_a (GTK_OBJECT_TYPE (item), item_type))
-    item = item->parent;
-
-  return item && gtk_type_is_a (GTK_OBJECT_TYPE (item), item_type) ? item : NULL;
-}
-
-/* read bstdefs.h on this */
-GnomeCanvasPoints*
-gnome_canvas_points_new0 (guint num_points)
-{
-  GnomeCanvasPoints *points;
-  guint i;
-  
-  g_return_val_if_fail (num_points > 1, NULL);
-  
-  points = gnome_canvas_points_new (num_points);
-  for (i = 0; i < num_points; i++)
-    {
-      points->coords[i] = 0;
-      points->coords[i + num_points] = 0;
-    }
-  
-  return points;
-}
-
-GnomeCanvasPoints*
-gnome_canvas_points_newv (guint num_points,
-			  ...)
-{
-  GnomeCanvasPoints *points;
-  guint i;
-  va_list args;
-  
-  g_return_val_if_fail (num_points > 1, NULL);
-  
-  va_start (args, num_points);
-  points = gnome_canvas_points_new (num_points);
-  for (i = 0; i < num_points * 2; i++)
-    points->coords[i] = va_arg (args, gdouble);
-  va_end (args);
-  
-  return points;
-}
-
-static void
-item_request_update_recurse (GnomeCanvasItem *item)
-{
-  g_return_if_fail (GNOME_IS_CANVAS_ITEM (item));
-  
-  gnome_canvas_item_request_update (item);
-  
-  if (GNOME_IS_CANVAS_GROUP (item))
-    {
-      GnomeCanvasGroup *group = GNOME_CANVAS_GROUP (item);
-      GList *list;
-      
-      for (list = group->item_list; list; list = list->next)
-	item_request_update_recurse (list->data);
-    }
-}
-
-void
-gnome_canvas_request_full_update (GnomeCanvas *canvas)
-{
-  g_return_if_fail (GNOME_IS_CANVAS (canvas));
-  
-  item_request_update_recurse (canvas->root);
-}
-
-guint
-gnome_canvas_item_get_stacking (GnomeCanvasItem *item)
-{
-  g_return_val_if_fail (GNOME_IS_CANVAS_ITEM (item), 0);
-  
-  if (item->parent)
-    {
-      GnomeCanvasGroup *parent = GNOME_CANVAS_GROUP (item->parent);
-      GList *list;
-      guint pos = 0;
-      
-      for (list = parent->item_list; list; list = list->next)
-	{
-	  if (list->data == item)
-	    return pos;
-	  pos++;
-	}
-    }
-  
-  return 0;
-}
-
-extern void
-gnome_canvas_item_keep_between (GnomeCanvasItem *between,
-				GnomeCanvasItem *item1,
-				GnomeCanvasItem *item2)
-{
-  g_return_if_fail (GNOME_IS_CANVAS_ITEM (between));
-  g_return_if_fail (GNOME_IS_CANVAS_ITEM (item1));
-  g_return_if_fail (GNOME_IS_CANVAS_ITEM (item2));
-  
-  if (between->parent && item1->parent == between->parent && item2->parent == between->parent)
-    {
-      guint n, i, z;
-      
-      n = gnome_canvas_item_get_stacking (item1);
-      i = gnome_canvas_item_get_stacking (item2);
-      z = gnome_canvas_item_get_stacking (between);
-      n = (n + i + (z > MIN (n, i))) / 2;
-      if (z < n)
-	gnome_canvas_item_raise (between, n - z);
-      else if (n < z)
-	gnome_canvas_item_lower (between, z - n);
-    }
-  else
-    g_warning ("gnome_canvas_item_keep_between() called for non-siblings");
-}
-
-extern void
-gnome_canvas_item_keep_above (GnomeCanvasItem *above,
-			      GnomeCanvasItem *item1,
-			      GnomeCanvasItem *item2)
-{
-  g_return_if_fail (GNOME_IS_CANVAS_ITEM (above));
-  g_return_if_fail (GNOME_IS_CANVAS_ITEM (item1));
-  g_return_if_fail (GNOME_IS_CANVAS_ITEM (item2));
-  
-  if (above->parent && item1->parent == above->parent && item2->parent == above->parent)
-    {
-      guint n, i, z;
-      
-      n = gnome_canvas_item_get_stacking (item1);
-      i = gnome_canvas_item_get_stacking (item2);
-      z = gnome_canvas_item_get_stacking (above);
-      n = MAX (n, i) + 1;
-      if (z < n)
-	gnome_canvas_item_raise (above, n - z);
-      else if (n < z)
-	gnome_canvas_item_lower (above, z - n);
-    }
-  else
-    g_warning ("gnome_canvas_item_keep_above() called for non-siblings");
-}
