@@ -1,19 +1,20 @@
 /* BEAST - Bedevilled Audio System
- * Copyright (C) 1998, 1999 Olaf Hoehmann and Tim Janik
+ * Copyright (C) 1998, 1999, 2000 Tim Janik and Red Hat, Inc.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU Lesser General
+ * Public License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 #include	"bstparam.h"
 
@@ -80,7 +81,6 @@ _GROUP_CALL (GtkWidget *group,
 	     gpointer   d)
 {
   static const gchar *names[] = {
-    "group_container",
     "group_prompt",
     "group_dial",
     "group_scale",
@@ -136,6 +136,8 @@ GROUP_FORM_BIG (GtkWidget *parent,
 #define GROUP_ADD_SCALE(g, w)             (_GROUP_ADD_NAMED_OBJECT ((g), "group_scale", (w)))
 #define GROUP_GET_ACTION(g)	          (GTK_WIDGET (g))
 #define GROUP_GET_PRE_ACTION(g)	          (_GROUP_GET_NAMED_WIDGET ((g), "group_pre_action"))
+#define GROUP_GET_POST_ACTION(g)	  (_GROUP_GET_NAMED_WIDGET ((g), "group_post_action"))
+#define GROUP_GET_PROMPT(g)	          (_GROUP_GET_NAMED_WIDGET ((g), "group_prompt"))
 static void
 GROUP_SET_TIP (GtkWidget   *group,
 	       const gchar *tip_text,
@@ -171,7 +173,7 @@ GROUP_DONE (GtkWidget *group)
   GtkTooltips *tt = NULL;
   gchar *tip_text, *tip_ref = NULL;
   GtkTable *table;
-  gboolean big, expandable;
+  gboolean big, expandable, dummy_scale = FALSE;
   guint row, n, c;
 
   g_return_val_if_fail (GTK_IS_WIDGET (group), NULL);
@@ -216,20 +218,42 @@ GROUP_DONE (GtkWidget *group)
 		      c, c + 1, row, row + 1,
 		      GTK_FILL, GTK_FILL,
 		      0, 0);
+  gtk_table_set_col_spacing (table, c, 2); /* seperate prompt from rest */
   c++;
   if (dial)
     gtk_table_attach (table, gtk_widget_get_toplevel (dial),
 		      c, c + 1, row, row + 1,
-		      GTK_FILL, GTK_FILL,
-		      5, 0);
+		      /* GTK_SHRINK | */ GTK_FILL, GTK_FILL,
+		      0, 0);
   c++;
-  gtk_table_attach (table,
-		    scale ? gtk_widget_get_toplevel (scale) : gtk_widget_new (GTK_TYPE_ALIGNMENT,
-									      "visible", TRUE,
-									      NULL),
-		    dial ? c : c - 1, c + 1, row, row + 1,
-		    GTK_EXPAND | GTK_FILL, 0,
-		    5, 0);
+  if (!scale)
+    {
+      scale = gtk_object_get_data (GTK_OBJECT (table), "dummy_scale");
+
+      /* need to have at least 1 (dummy) scale per table
+       * to eat up expanding space
+       */
+      if (!scale)
+	{
+	  scale = gtk_widget_new (GTK_TYPE_ALIGNMENT,
+				  "visible", TRUE,
+				  NULL);
+	  gtk_object_set_data (GTK_OBJECT (table), "dummy_scale", scale);
+	}
+      else
+	scale = NULL;
+      dummy_scale = TRUE;
+    }
+  if (scale)
+    {
+      gtk_table_attach (table,
+			gtk_widget_get_toplevel (scale),
+			dial ? c : c - 1, c + 1, row, row + 1,
+			GTK_EXPAND | GTK_FILL, 0,
+			0, 0);
+      if (dummy_scale)
+	scale = NULL;
+    }
   c++;
   /* stuff action with pre- and post- widgets closely together */
   any = gtk_widget_get_toplevel (action);
@@ -271,8 +295,9 @@ GROUP_DONE (GtkWidget *group)
 			  NULL);
   gtk_table_attach (table, any,
 		    n, c + 1, row, row + 1,
-		    GTK_FILL, GTK_FILL,
+		    GTK_SHRINK | GTK_FILL, GTK_FILL,
 		    0, 0);
+  gtk_table_set_col_spacing (table, c - 1, 2); /* seperate action from rest */
   
   /* set tooltips */
   if (tt)
@@ -293,8 +318,9 @@ GROUP_DONE (GtkWidget *group)
 
   return group;
 }
-#define GROUP_DESTROY(g)       (_GROUP_CALL ((g), gtk_widget_destroy, NULL))
-#define GROUP_ENSURE_STYLES(g) (_GROUP_CALL ((g), gtk_widget_ensure_style, NULL))
+#define GROUP_DESTROY(g)          (_GROUP_CALL ((g), gtk_widget_destroy, NULL))
+#define GROUP_ENSURE_STYLES(g)    (_GROUP_CALL ((g), gtk_widget_ensure_style, NULL))
+#define GROUP_SET_SENSITIVE(g, s) (_GROUP_CALL ((g), (s) ? gtk_widget_make_sensitive : gtk_widget_make_insensitive, NULL))
 
 
 /* --- functions --- */
@@ -302,7 +328,7 @@ static void
 bst_string_toggle (GtkToggleButton *tb,
 		   GtkWidget       *widget)
 {
-  gtk_widget_set_sensitive (widget, tb->active);
+  gtk_editable_set_editable (GTK_EDITABLE (widget), tb->active);
   if (tb->active)
     gtk_widget_grab_focus (widget);
 }
@@ -322,42 +348,81 @@ bst_param_gtk_update (BstParam *bparam)
 }
 
 static gboolean
-bst_entry_key_press (GtkWidget   *entry,
+bst_entry_key_press (GtkEntry    *entry,
 		     GdkEventKey *event,
 		     BstParam    *bparam)
 {
-  gboolean is_tab = FALSE;
-
-  if (!gtk_type_is_a (GTK_OBJECT_TYPE (entry), GTK_TYPE_SPIN_BUTTON))
+  GtkEditable *editable = GTK_EDITABLE (entry);
+  gboolean intercept = FALSE;
+  
+  if (event->state & GDK_MOD1_MASK)
     switch (event->keyval)
       {
-      case GDK_Tab:
-      case GDK_ISO_Left_Tab:
-	is_tab = TRUE;
-	/* fall through */
-      case GDK_Up:
-      case GDK_KP_Up:
-      case GDK_Down:
-      case GDK_KP_Down:
-	/* special case clue hunter entries */
-	if (gtk_clue_hunter_from_entry (entry))
-	  {
-	    if (!is_tab && !gtk_grab_get_current ())
-	      bst_param_gtk_changed (bparam);
-	  }
-	else
-	  bst_param_gtk_changed (bparam);
+      case 'b':	/* check gtk_move_backward_word() */
+	intercept = editable->current_pos <= 0;
+	break;
+      case 'd': /* gtk_delete_forward_word() */
+	intercept = TRUE;
+	break;
+      case 'f': /* check gtk_move_forward_word() */
+	intercept = editable->current_pos >= entry->text_length;
 	break;
       default:
 	break;
       }
-
-  if (event->keyval == 'f' && (event->state & GDK_MOD1_MASK))
+  
+  if (intercept)
     gtk_signal_emit_stop_by_name (GTK_OBJECT (entry), "key_press_event");
-
+  
   return FALSE;
 }
 
+static void
+bst_param_update_clue_hunter (BstParam *bparam)
+{
+  GtkWidget *group = bparam->group;
+  BseParam *param = &bparam->param;
+  BseParamSpec *pspec = param->pspec;
+  GtkWidget *action = GROUP_GET_ACTION (group);
+  GtkClueHunter *ch = gtk_object_get_user_data (GTK_OBJECT (action));
+
+  g_return_if_fail (pspec->type == BSE_TYPE_PARAM_ITEM && GTK_IS_CLUE_HUNTER (ch));
+  
+  if (bse_type_is_a (pspec->s_item.item_type, BSE_TYPE_SAMPLE))
+    {
+      GList *free_list = bst_sample_repo_list_sample_locs (), *list;
+      
+      gtk_clue_hunter_remove_matches (ch, "*");
+      
+      for (list = free_list; list; list = list->next)
+	{
+	  BstSampleLoc *loc = list->data;
+	  
+	  /* FIXME: showing the repo as well would be nice */
+	  gtk_clue_hunter_add_string (ch, loc->name);
+	}
+      g_list_free (free_list);
+    }
+  else if (bparam->is_object && BSE_IS_ITEM (bparam->owner) &&
+	   bse_type_is_a (pspec->s_item.item_type, BSE_TYPE_ITEM))
+    {
+      BseItem *item = BSE_ITEM (bparam->owner);
+      BseProject *project = bse_item_get_project (item);
+      GList *nick_list, *list;
+      
+      gtk_clue_hunter_remove_matches (ch, "*");
+
+      nick_list = bse_project_list_nick_paths (project, pspec->s_item.item_type);
+      
+      for (list = nick_list; list; list = list->next)
+	{
+	  gtk_clue_hunter_add_string (ch, list->data);
+	  g_free (list->data);
+	}
+      g_list_free (nick_list);
+    }
+}
+
 static void
 bst_bparam_bse_changed (BstParam     *bparam,
 			BseParamSpec *pspec)
@@ -366,15 +431,6 @@ bst_bparam_bse_changed (BstParam     *bparam,
     bst_param_get (bparam);
 }
 
-#ifdef DEBUG_ADJUSTMENT
-static void
-debug_adjustment (GtkAdjustment *adjustment,
-		  BstParam      *bparam)
-{
-  g_print ("adjustment changed: %f (%u)\n", adjustment->value, bparam->locked);
-}
-#endif
-
 void
 bst_param_destroy (BstParam *bparam)
 {
@@ -429,6 +485,14 @@ bparam_reset_object (BstParam *bparam)
   bst_param_set_object (bparam, NULL);
 }
 
+static void
+hscale_size_request (GtkWidget      *scale,
+		     GtkRequisition *requisition)
+{
+  requisition->width = GTK_SCALE_CLASS (GTK_OBJECT (scale)->klass)->slider_length;
+  requisition->width += 2 * scale->style->klass->xthickness + 2;
+}
+
 void
 bst_param_set_object (BstParam  *bparam,
 		      BseObject *object)
@@ -471,6 +535,7 @@ BstParam*
 bst_param_create (gpointer      owner,
 		  BseType	owner_type,
 		  BseParamSpec *pspec,
+		  GQuark        param_group,
 		  GtkWidget    *parent,
 		  GtkTooltips  *tooltips)
 {
@@ -516,29 +581,37 @@ bst_param_create (gpointer      owner,
     }
   bparam->locked = 1;
   bse_type_class_ref (owner_type);
+  bparam->editable = TRUE;
   
-  parent_container = gtk_object_get_data_by_id (GTK_OBJECT (parent), pspec->any.param_group ? pspec->any.param_group : null_group);
+  parent_container = gtk_object_get_data_by_id (GTK_OBJECT (parent), param_group ? param_group : null_group);
   if (!parent_container ||
       GTK_OBJECT_DESTROYED (parent_container) ||
       !GTK_IS_CONTAINER (parent_container))
     {
       GtkWidget *any;
       
-      any = GROUP_PARENT_CREATE (tooltips, pspec->any.param_group ? 5 : 0);
+      any = GROUP_PARENT_CREATE (tooltips, param_group ? 5 : 0);
       parent_container = any;
       gtk_widget_ref (any);
       gtk_object_set_data_by_id_full (GTK_OBJECT (parent),
-				      pspec->any.param_group ? pspec->any.param_group : null_group,
+				      param_group ? param_group : null_group,
 				      any,
 				      (GtkDestroyNotify) gtk_widget_unref);
-      if (pspec->any.param_group)
+      if (param_group)
 	any = gtk_widget_new (GTK_TYPE_FRAME,
 			      "visible", TRUE,
-			      "label", g_quark_to_string (pspec->any.param_group),
+			      "label", g_quark_to_string (param_group),
 			      "child", any,
 			      NULL);
       if (GTK_IS_BOX (parent))
 	gtk_box_pack_start (GTK_BOX (parent), any, FALSE, TRUE, 0);
+      else if (GTK_IS_WRAP_BOX (parent))
+	gtk_container_add_with_args (GTK_CONTAINER (parent), any,
+				     "hexpand", TRUE,
+				     "hfill", TRUE,
+				     "vexpand", FALSE,
+				     "vfill", TRUE,
+				     NULL);
       else
 	gtk_container_add (GTK_CONTAINER (parent), any);
     }
@@ -604,24 +677,20 @@ bst_param_create (gpointer      owner,
 				       "value-changed",
 				       bst_param_gtk_changed,
 				       (GtkObject*) bparam);
-#ifdef DEBUG_ADJUSTMENT
-      gtk_signal_connect (GTK_OBJECT (adjustment), "value-changed", debug_adjustment, bparam);
-#endif
-      
       spinner = gtk_spin_button_new (adjustment, 0, digits);
       if (pspec->any.flags & BSE_PARAM_HINT_DIAL)
-	{
-	  dial = gtk_widget_new (GTK_TYPE_LABEL,
-				 "visible", TRUE,
-				 "label", "|<DIAL>|", /* FIXME: we need a real dial */
-				 NULL);
-	}
+	dial = gtk_widget_new (GTK_TYPE_LABEL,
+			       "visible", TRUE,
+			       "label", "(dial)", /* FIXME: we need a real dial */
+			       NULL);
       if (pspec->any.flags & BSE_PARAM_HINT_SCALE
 	  || pspec->any.flags & BSE_PARAM_HINT_DIAL) /* FIXME: we need a real dial */
-	{
-	  scale = gtk_hscale_new (adjustment);
-	  gtk_scale_set_draw_value (GTK_SCALE (scale), FALSE);
-	}
+	scale = gtk_widget_new (GTK_TYPE_HSCALE,
+				"visible", TRUE,
+				"adjustment", adjustment,
+				"draw_value", FALSE,
+				"signal_after::size_request", hscale_size_request, NULL,
+				NULL);
       
       gtk_object_unref (GTK_OBJECT (adjustment));
     }
@@ -641,7 +710,6 @@ bst_param_create (gpointer      owner,
       action = gtk_widget_new (radio ? BST_TYPE_FREE_RADIO_BUTTON : GTK_TYPE_CHECK_BUTTON,
 			       "visible", TRUE,
 			       "label", name,
-			       "sensitive", !read_only,
 			       "object_signal::clicked", bst_param_gtk_changed, bparam,
 			       NULL);
       gtk_misc_set_alignment (GTK_MISC (GTK_BIN (action)->child), 0, 0.5);
@@ -684,15 +752,15 @@ bst_param_create (gpointer      owner,
 			       "label", name,
 			       "justify", GTK_JUSTIFY_LEFT,
 			       "xalign", 0.0,
-			       "sensitive", !read_only,
 			       NULL);
       action = spinner ? spinner : gtk_entry_new ();
       gtk_widget_set (action,
 		      "visible", TRUE,
 		      "width", width,
-		      "object_signal::activate", bst_param_gtk_changed, bparam,
-		      "sensitive", !read_only,
 		      "signal::key_press_event", bst_entry_key_press, bparam,
+		      "object_signal::activate", bst_param_gtk_changed, bparam,
+		      "signal_after::activate", gtk_toplevel_activate_default, NULL,
+		      spinner ? NULL : "object_signal::focus_out_event", bst_param_gtk_update, bparam,
 		      NULL);
       if (!spinner)
 	gtk_widget_set (action,
@@ -701,17 +769,9 @@ bst_param_create (gpointer      owner,
       group = GROUP_FORM (parent_container, action, expandable);
       GROUP_ADD_PROMPT (group, prompt);
       if (scale)
-	{
-	  gtk_widget_show (scale);
-	  gtk_widget_set_sensitive (scale, !read_only);
-	  GROUP_ADD_SCALE (group, scale);
-	}
+	GROUP_ADD_SCALE (group, scale);
       if (dial)
-	{
-	  gtk_widget_show (dial);
-	  gtk_widget_set_sensitive (dial, !read_only);
-	  GROUP_ADD_DIAL (group, dial);
-	}
+	GROUP_ADD_DIAL (group, dial);
       GROUP_SET_TIP (group, tooltip, NULL);
       widget_group = GROUP_DONE (group);
       break;
@@ -727,7 +787,6 @@ bst_param_create (gpointer      owner,
       action = gtk_option_menu_new ();
       gtk_widget_set (action,
 		      "visible", TRUE,
-		      "sensitive", !read_only && ev,
 		      NULL);
       if (ev)
 	{
@@ -781,14 +840,13 @@ bst_param_create (gpointer      owner,
 			       "label", name,
 			       "justify", GTK_JUSTIFY_LEFT,
 			       "xalign", 0.0,
-			       "sensitive", !read_only,
 			       NULL);
       action = gtk_widget_new (GTK_TYPE_ENTRY,
 			       "visible", TRUE,
-			       "object_signal::focus_out_event", bst_param_gtk_update, bparam,
-			       "object_signal::activate", bst_param_gtk_changed, bparam,
 			       "signal::key_press_event", bst_entry_key_press, bparam,
-			       "sensitive", !read_only,
+			       "object_signal::activate", bst_param_gtk_changed, bparam,
+			       "signal_after::activate", gtk_toplevel_activate_default, NULL,
+			       "object_signal::focus_out_event", bst_param_gtk_update, bparam,
 			       NULL);
       group = GROUP_FORM (parent_container, action, TRUE);
       GROUP_ADD_PROMPT (group, prompt);
@@ -799,18 +857,18 @@ bst_param_create (gpointer      owner,
 				       "can_focus", FALSE,
 				       "width", 10,
 				       "height", 10,
-				       "parent", gtk_widget_new (GTK_TYPE_ALIGNMENT,
-								 "visible", !read_only,
+				       "parent", gtk_widget_new (GTK_TYPE_ALIGNMENT, /* don't want vexpand */
+								 "visible", TRUE,
 								 "xscale", 0.0,
 								 "yscale", 0.0,
 								 "xalign", 0.0,
-								 "width", 13,
+								 "width", 10 + 3,
 								 NULL),
 				       "object_signal::clicked", bst_param_gtk_changed, bparam,
 				       "signal::clicked", bst_string_toggle, action,
 				       NULL);
-	  bst_string_toggle (GTK_TOGGLE_BUTTON (pre_action), action);
 	  GROUP_ADD_PRE_ACTION (group, pre_action);
+	  bst_string_toggle (GTK_TOGGLE_BUTTON (pre_action), action);
 	}
       GROUP_SET_TIP (group, tooltip, NULL);
       widget_group = GROUP_DONE (group);
@@ -821,13 +879,12 @@ bst_param_create (gpointer      owner,
 			       "label", name,
 			       "justify", GTK_JUSTIFY_LEFT,
 			       "xalign", 0.0,
-			       "sensitive", !read_only,
 			       NULL);
       frame = gtk_widget_new (GTK_TYPE_FRAME,
 			      "visible", TRUE,
 			      "label", NULL,
-			      "shadow", GTK_SHADOW_OUT,
-			      "border_width", 5,
+			      "shadow", GTK_SHADOW_IN,
+			      "border_width", 0,
 			      NULL);
       dot_data = g_new0 (DotAreaData, 1);
       action = gtk_widget_new (GTK_TYPE_DRAWING_AREA,
@@ -862,15 +919,15 @@ bst_param_create (gpointer      owner,
 			       "label", name,
 			       "justify", GTK_JUSTIFY_LEFT,
 			       "xalign", 0.0,
-			       "sensitive", !read_only,
 			       NULL);
       action = gtk_widget_new (GTK_TYPE_ENTRY,
 			       "visible", TRUE,
-			       "width", 160,
-			       "object_signal::focus_out_event", bst_param_gtk_update, bparam,
-			       "object_signal::activate", bst_param_gtk_changed, bparam,
 			       "signal::key_press_event", bst_entry_key_press, bparam,
-			       "sensitive", !read_only,
+			       "object_signal::activate", bst_param_gtk_changed, bparam,
+			       "signal_after::activate", gtk_toplevel_activate_default, NULL,
+			       "object_signal::focus_out_event", bst_param_gtk_update, bparam,
+			       "object_signal::grab_focus", bst_param_update_clue_hunter, bparam,
+			       "width", 160,
 			       NULL);
       group = GROUP_FORM (parent_container, action, TRUE);
       GROUP_ADD_PROMPT (group, prompt);
@@ -912,10 +969,13 @@ bst_param_update (BstParam *bparam)
   GtkWidget *group = bparam->group;
   BseParam *param = &bparam->param;
   BseParamSpec *pspec = param->pspec;
+  gboolean read_only = (pspec->any.flags & BSE_PARAM_HINT_RDONLY) != 0;
+
+  GROUP_SET_SENSITIVE (group, !read_only && bparam->editable);
   
   switch (pspec->type)
     {
-      GtkWidget *action, *pre_action, *any;
+      GtkWidget *action, *prompt, *pre_action, *any;
       gchar *string;
       
     case BSE_TYPE_PARAM_BOOL:
@@ -942,16 +1002,6 @@ bst_param_update (BstParam *bparam)
       if (!g_str_equal (gtk_entry_get_text (GTK_ENTRY (action)), string))
 	{
 	  gtk_entry_set_text (GTK_ENTRY (action), string);
-#if 0
-	  GtkRequisition requisition;
-	  
-	  gtk_widget_size_request (action, &requisition);
-	  if (MAX (requisition.width, action->requisition.width) > action->allocation.width)
-	    gtk_widget_set_usize (action,
-				  MAX (requisition.width, action->requisition.width),
-				  -1);
-#endif
-	  // gtk_entry_set_position (GTK_ENTRY (action), 0);
 	  if (GTK_IS_SPIN_BUTTON (action))
 	    gtk_spin_button_update (GTK_SPIN_BUTTON (action));
 	}
@@ -961,10 +1011,7 @@ bst_param_update (BstParam *bparam)
       action = GROUP_GET_ACTION (group);
       string = param->value.v_string;
       if (!string || !g_str_equal (gtk_entry_get_text (GTK_ENTRY (action)), string))
-	{
-	  gtk_entry_set_text (GTK_ENTRY (action), string ? string : "");
-	  // gtk_entry_set_position (GTK_ENTRY (action), 0);
-	}
+	gtk_entry_set_text (GTK_ENTRY (action), string ? string : "");
       pre_action = GROUP_GET_PRE_ACTION (group);
       if (pre_action)
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (pre_action), string != NULL);
@@ -975,37 +1022,16 @@ bst_param_update (BstParam *bparam)
       break;
     case BSE_TYPE_PARAM_ITEM:
       action = GROUP_GET_ACTION (group);
-      string = param->value.v_item ? BSE_OBJECT_NAME (param->value.v_item) : NULL;
+      string = param->value.v_item ? bse_item_make_nick_path (param->value.v_item) : NULL;
       if (!bse_string_equals (gtk_entry_get_text (GTK_ENTRY (action)), string))
-	{
-	  gtk_entry_set_text (GTK_ENTRY (action), string ? string : "");
-	  // gtk_entry_set_position (GTK_ENTRY (action), 0);
-	}
+	gtk_entry_set_text (GTK_ENTRY (action), string ? string : "");
       any = gtk_object_get_user_data (GTK_OBJECT (action));
-      if (any)
-	{
-	  GtkClueHunter *ch = GTK_CLUE_HUNTER (any);
-	  
-	  if (pspec->s_item.item_type == BSE_TYPE_SAMPLE)
-	    {
-	      GList *free_list = bst_sample_repo_list_sample_locs (), *list;
-	      
-	      gtk_clue_hunter_remove_matches (ch, "*");
-	      
-	      for (list = free_list; list; list = list->next)
-		{
-		  BstSampleLoc *loc = list->data;
-		  
-		  /* FIXME: showing the repo as well would be nice */
-		  gtk_clue_hunter_add_string (ch, loc->name);
-		}
-	      g_list_free (free_list);
-	    }
-	}
       break;
     case BSE_TYPE_PARAM_ENUM:
       action = GROUP_GET_ACTION (group);
       any = gtk_option_menu_get_menu (GTK_OPTION_MENU (action));
+      prompt = GROUP_GET_PROMPT (group);
+      gtk_widget_set_sensitive (prompt, GTK_WIDGET_IS_SENSITIVE (prompt) && pspec->s_enum.enum_class->values);
       if (any)
 	{
 	  GList *list;
@@ -1054,7 +1080,7 @@ bst_param_apply (BstParam *bparam,
       gchar *string;
       BseTime time_data;
       guint base;
-      guint note_data;
+      gint note_data;
 
     case BSE_TYPE_PARAM_BOOL:
       action = GROUP_GET_ACTION (group);
@@ -1141,43 +1167,51 @@ bst_param_apply (BstParam *bparam,
     case BSE_TYPE_PARAM_ITEM:
       action = GROUP_GET_ACTION (group);
       string = bse_strdup_stripped (gtk_entry_get_text (GTK_ENTRY (action)));
-      if (string && bparam->is_object)
+      if (string && bparam->is_object && BSE_IS_ITEM (bparam->owner))
 	{
-	  GList *list, *free_list;
-	  gboolean try_further = TRUE;
+	  BseProject *project = bse_item_get_project (BSE_ITEM (bparam->owner));
+	  BseItem *item = NULL;
 
-	  free_list = bse_objects_list_by_name (pspec->s_item.item_type, string);
-	  for (list = free_list; list; list = list->next)
+	  /* check whether this is a nick path */
+	  if (!item && strchr (string, '.'))
 	    {
-	      BseSuper *super = list->data;
+	      item = bse_project_item_from_nick_path (project, string);
 
-	      if (bse_super_get_project (super) == bse_item_get_project (BSE_ITEM (bparam->owner)))
-		{
-		  dirty += bse_param_set_item (param, BSE_ITEM (super));
-		  try_further = FALSE;
-		  break;
-		}
+	      if (item && !bse_type_is_a (BSE_OBJECT_TYPE (item), pspec->s_item.item_type))
+		item = NULL;
+	    }
+	  else if (!item) /* try generic lookup for pure name (brute force actually) */
+	    {
+	      GList *list, *free_list = bse_objects_list_by_name (pspec->s_item.item_type, string);
+	      
+	      for (list = free_list; list; list = list->next)
+		if (bse_item_get_project (list->data) == project)
+		  {
+		    item = BSE_ITEM (list->data);
+		    break;
+		  }
+	      g_list_free (free_list);
 	    }
-	  g_list_free (free_list);
 
-	  if (try_further && pspec->s_item.item_type == BSE_TYPE_SAMPLE)
+	  /* check whether this refers to an on-disk sample that we can demand load */
+	  if (!item && pspec->s_item.item_type == BSE_TYPE_SAMPLE)
 	    {
 	      BstSampleLoc *loc = bst_sample_repo_find_sample_loc (string);
-
+	      
 	      if (loc)
 		{
-		  BseSample *sample;
-
-		  g_message ("demand load: %s (%s)", loc->name, loc->repo->name);
-
-		  sample = bst_sample_repo_load_sample (loc,
-							bse_item_get_project (BSE_ITEM (bparam->owner)));
-		  dirty += bse_param_set_item (param, BSE_ITEM (sample));
+		  g_message ("demand loading: %s (%s)", loc->name, loc->repo->name);
+		  
+		  item = (BseItem*) bst_sample_repo_load_sample (loc, project);
 		}
 	    }
 
-	  dirty += 1;
+	  /* ok, found one or giving up */
+	  dirty += bse_param_set_item (param, item);
 	  g_free (string);
+
+	  /* enforce redisplay of the entry's string with the correct name */
+	  dirty += 1;
 	}
       else
 	dirty += bse_param_set_item (param, NULL);
@@ -1240,6 +1274,17 @@ bst_param_set (BstParam *bparam)
     bparam->locked = 0;
 }
 
+void
+bst_param_reset (BstParam *bparam)
+{
+  g_return_if_fail (BSE_IS_PARAM (bparam));
+
+  bparam->locked++;
+  bse_param_reset_value (&bparam->param);
+  bst_param_update (bparam);
+  bparam->locked = 0;
+}
+
 gboolean
 bst_param_set_from_other (BstParam *bparam,
 			  BseParam *src_param)
@@ -1269,14 +1314,11 @@ bst_param_set_editable (BstParam *bparam,
 {
   g_return_if_fail (BSE_IS_PARAM (bparam));
 
-  if (bparam->group)
+  editable = editable != FALSE;
+  if (bparam->editable != editable)
     {
-      GtkWidget *action = GROUP_GET_ACTION (bparam->group);
-      
-      if (GTK_IS_EDITABLE (action))
-	gtk_editable_set_editable (GTK_EDITABLE (action), editable);
-      else
-	gtk_widget_set_sensitive (action, editable);
+      bparam->editable = editable;
+      bst_param_update (bparam);
     }
 }
 
