@@ -1,19 +1,20 @@
 /* BEAST - Bedevilled Audio System
- * Copyright (C) 1998, 1999 Olaf Hoehmann and Tim Janik
+ * Copyright (C) 1998, 1999, 2000 Tim Janik and Red Hat, Inc.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU Lesser General
+ * Public License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 #include "bstparamview.h"
 
@@ -50,7 +51,7 @@ bst_param_view_get_type (void)
 	(GtkClassInitFunc) NULL,
       };
       
-      param_view_type = gtk_type_unique (GTK_TYPE_VBOX, &param_view_info);
+      param_view_type = gtk_type_unique (GTK_TYPE_VWRAP_BOX, &param_view_info);
     }
   
   return param_view_type;
@@ -64,7 +65,7 @@ bst_param_view_class_init (BstParamViewClass *class)
   object_class = GTK_OBJECT_CLASS (class);
 
   bst_param_view_class = class;
-  parent_class = gtk_type_class (GTK_TYPE_VBOX);
+  parent_class = gtk_type_class (GTK_TYPE_VWRAP_BOX);
 
   object_class->destroy = bst_param_view_destroy;
 }
@@ -211,21 +212,19 @@ bst_param_view_rebuild (BstParamView *param_view)
 
   if (!param_view->object)
     return;
-
-  param_box = gtk_widget_new (GTK_TYPE_VBOX,
-			      "visible", TRUE,
-			      "parent", param_view,
-			      NULL);
-
+  
+  param_box = GTK_WIDGET (param_view);
+  gtk_widget_set (param_box,
+		  "visible", TRUE,
+		  "homogeneous", FALSE,
+		  "border_width", 5,
+		  "hspacing", 5,
+		  "aspect_ratio", 0.0,
+		  NULL);
+  
   object = BSE_OBJECT (param_view->object);
   class = BSE_OBJECT_GET_CLASS (object);
   
-  gtk_widget_set (param_box,
-		  "homogeneous", FALSE,
-		  "spacing", 0,
-		  "border_width", 5,
-		  NULL);
-  
   /* parameter fields, per bse class
    */
   while (class && bse_type_is_a (BSE_CLASS_TYPE (class), param_view->base_type))
@@ -237,47 +236,48 @@ bst_param_view_rebuild (BstParamView *param_view)
   
   for (slist = class_list; slist; slist = slist->next)
     {
-      BseParamSpec **pspec_p;
+      guint i, n;
       
       class = slist->data;
-      
-      pspec_p = class->param_specs;
-      if (pspec_p)
-	while (*pspec_p)
-	  {
-	    if ((*pspec_p)->any.flags & BSE_PARAM_SERVE_GUI &&
-		(*pspec_p)->any.flags & BSE_PARAM_READABLE &&
-		(!param_view->reject_pattern ||
-		 !g_pattern_match_string (param_view->reject_pattern, (*pspec_p)->any.name)) &&
-		(!param_view->match_pattern ||
-		 g_pattern_match_string (param_view->match_pattern, (*pspec_p)->any.name)))
-	      {
-		BstParam *bparam;
-		
-		bparam = bst_param_create (object,
-					   BSE_TYPE_OBJECT,
-					   *pspec_p,
-					   param_box,
-					   GTK_TOOLTIPS (param_view->tooltips));
-		param_view->bparams = g_slist_prepend (param_view->bparams, bparam);
-	      }
-	    pspec_p++;
-	  }
+      n = bse_object_class_get_n_param_specs (class);
+      for (i = 0; i < n; i++)
+	{
+	  GQuark param_group;
+	  BseParamSpec *pspec = bse_object_class_get_param_spec (class, i, &param_group);
+	  
+	  if (pspec->any.flags & BSE_PARAM_SERVE_GUI &&
+	      pspec->any.flags & BSE_PARAM_READABLE &&
+	      (!param_view->reject_pattern ||
+	       !g_pattern_match_string (param_view->reject_pattern, pspec->any.name)) &&
+	      (!param_view->match_pattern ||
+	       g_pattern_match_string (param_view->match_pattern, pspec->any.name)))
+	    {
+	      BstParam *bparam;
+	      
+	      bparam = bst_param_create (object,
+					 BSE_TYPE_OBJECT,
+					 pspec,
+					 param_group,
+					 param_box,
+					 GTK_TOOLTIPS (param_view->tooltips));
+	      param_view->bparams = g_slist_prepend (param_view->bparams, bparam);
+	    }
+	}
     }
   g_slist_free (class_list);
-
-  /* FIXME: work around a gtk bug 
+  
+  /* FIXME: work around a gtk bug (<= 1.2.6)
    */
   if (GTK_WIDGET_REALIZED (param_view) && !GTK_CHECK_VERSION (1, 2, 7))
     {
       GtkWidget *toplevel = gtk_widget_get_toplevel (GTK_WIDGET (param_view));
-
+      
       gtk_container_check_resize (GTK_CONTAINER (toplevel));
       gtk_container_check_resize (GTK_CONTAINER (toplevel));
       gtk_container_check_resize (GTK_CONTAINER (toplevel));
       gtk_widget_queue_resize (toplevel);
     }
-
+  
   bst_param_view_update (param_view);
 }
 
