@@ -1,18 +1,18 @@
 /* BEAST - Bedevilled Audio System
- * Copyright (C) 1998, 1999 Olaf Hoehmann and Tim Janik
+ * Copyright (C) 1998, 1999, 2000 Olaf Hoehmann and Tim Janik
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this program; if not, write to the Free Software
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
  */
 #ifndef __BST_PATTERN_EDITOR_H__
@@ -202,7 +202,7 @@ struct _BstPatternEditor
   /* selection */
   guint		in_selection : 1;
   guint		selection_subtract : 1;
-  guint32	*saved_selection;
+  guint32      *saved_selection;
   guint16	selection_channel;
   guint16	selection_row;
   guint		selection_timer;
@@ -279,16 +279,12 @@ gint	   bst_pattern_editor_get_cell	      (BstPatternEditor		*pe,
 					       gint			*row_p);
 void	   bst_pattern_editor_set_focus	      (BstPatternEditor		*pe,
 					       guint			 channel,
-					       guint			 row);
+					       guint			 row,
+					       gboolean                  reset_selection);
 void	   bst_pattern_editor_adjust_sas      (BstPatternEditor		*pe,
 					       gboolean			 check_bounds);
 void	   bst_pattern_editor_set_octave      (BstPatternEditor		*pe,
 					       gint			 octave);
-void	   bst_pattern_editor_class_clear_keys(BstPatternEditorClass	*pe_class);
-void	   bst_pattern_editor_class_set_key   (BstPatternEditorClass	*pe_class,
-					       guint16			 keyval,
-					       guint16			 modifier,
-					       BstPEActionType		 pe_action);
 void	   bst_pattern_editor_mark_row	      (BstPatternEditor		*pe,
 					       gint			 row);
 void	   bst_pattern_editor_dfl_stepper     (BstPatternEditor		*pe,
@@ -299,6 +295,16 @@ void bst_pattern_editor_set_effect_hooks   (BstPatternEditor		*pe,
 					    BstPatternEffectAreaDraw	 ea_draw,
 					    gpointer			 user_data,
 					    GtkDestroyNotify		 ea_destroy);
+void	   bst_pattern_editor_class_clear_keys(BstPatternEditorClass	*pe_class);
+void	   bst_pattern_editor_class_set_key   (BstPatternEditorClass	*pe_class,
+					       guint16			 keyval,
+					       guint16			 modifier,
+					       BstPEActionType		 pe_action);
+GString*   bst_pattern_editor_class_keydump   (BstPatternEditorClass	*pe_class);
+
+
+/* --- selections --- */
+void bst_pattern_editor_reset_selection  (BstPatternEditor *pe);
 
 
 
