@@ -1,18 +1,18 @@
 /* BEAST - Bedevilled Audio System
- * Copyright (C) 1998, 1999 Olaf Hoehmann and Tim Janik
+ * Copyright (C) 1998, 1999, 2000 Olaf Hoehmann and Tim Janik
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this program; if not, write to the Free Software
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
  */
 #include "bstpatternview.h"
@@ -33,9 +33,9 @@ static gboolean	bst_pattern_view_can_operate	(BstItemView		*item_view,
 
 /* --- pattern ops --- */
 static BstItemViewOp pattern_view_ops[] = {
-  { "Add",		BST_OP_PATTERN_ADD,	},
-  { "Delete",		BST_OP_PATTERN_DELETE,	},
-  { "Editor...",	BST_OP_PATTERN_EDITOR,	},
+  { "Add",		BST_OP_PATTERN_ADD,	BST_ICON_PATTERN,	},
+  { "Delete",		BST_OP_PATTERN_DELETE,	BST_ICON_TRASHCAN,	},
+  { "Editor...",	BST_OP_PATTERN_EDITOR,	BST_ICON_PATTERN_TOOL,	},
 };
 static guint n_pattern_view_ops = sizeof (pattern_view_ops) / sizeof (pattern_view_ops[0]);
 
@@ -140,16 +140,18 @@ bst_pattern_view_operate (BstItemView *item_view,
   
   switch (op)
     {
-      BsePattern *pattern;
       BseItem *item;
       
     case BST_OP_PATTERN_ADD:
       item = bse_container_new_item (BSE_CONTAINER (song), BSE_TYPE_PATTERN, NULL);
+      bse_pattern_group_insert_pattern (bse_song_get_default_pattern_group (song),
+					BSE_PATTERN (item),
+					-1);
       bst_item_view_select (item_view, item);
       break;
     case BST_OP_PATTERN_DELETE:
-      pattern = (BsePattern*) bst_item_view_get_current (BST_ITEM_VIEW (pattern_view));
-      bse_song_delete_pattern (song, pattern);
+      item = bst_item_view_get_current (BST_ITEM_VIEW (pattern_view));
+      bse_container_remove_item (BSE_CONTAINER (song), item);
       break;
     case BST_OP_PATTERN_EDITOR:
       popup_pattern_dialog (pattern_view);
