@@ -1,18 +1,18 @@
 /* BEAST - Bedevilled Audio System
- * Copyright (C) 1998, 1999 Olaf Hoehmann and Tim Janik
+ * Copyright (C) 1998, 1999, 2000 Olaf Hoehmann and Tim Janik
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this program; if not, write to the Free Software
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
  */
 #ifndef __BST_SONG_SHELL_H__
@@ -21,8 +21,9 @@
 #include	"bstdefs.h"
 #include	"bstsupershell.h"
 #include	"bstparamview.h"
-#include	"bstpatternview.h"
 #include	"bstinstrumentview.h"
+#include	"bstpatternview.h"
+#include	"bstplaylist.h"
 
 
 #ifdef __cplusplus
@@ -48,8 +49,9 @@ struct _BstSongShell
   BstSuperShell	parent_object;
 
   BstParamView   *param_view;
-  BstItemView    *pattern_view;
   BstItemView    *instrument_view;
+  BstItemView    *pattern_view;
+  GtkWidget      *play_list;
 
   GtkTooltips	*tooltips;
 };
