@@ -47,7 +47,8 @@ enum {
   CHILD_ARG_HEXPAND,
   CHILD_ARG_HFILL,
   CHILD_ARG_VEXPAND,
-  CHILD_ARG_VFILL
+  CHILD_ARG_VFILL,
+  CHILD_ARG_WRAPPED
 };
 
 
@@ -171,6 +172,8 @@ gtk_wrap_box_class_init (GtkWrapBoxClass *class)
 				    GTK_TYPE_BOOL, GTK_ARG_READWRITE, CHILD_ARG_VEXPAND);
   gtk_container_add_child_arg_type ("GtkWrapBox::vfill",
 				    GTK_TYPE_BOOL, GTK_ARG_READWRITE, CHILD_ARG_VFILL);
+  gtk_container_add_child_arg_type ("GtkWrapBox::wrapped",
+				    GTK_TYPE_BOOL, GTK_ARG_READWRITE, CHILD_ARG_WRAPPED);
 }
 
 static void
@@ -274,10 +277,10 @@ gtk_wrap_box_set_child_arg (GtkContainer *container,
 			    guint         arg_id)
 {
   GtkWrapBox *wbox = GTK_WRAP_BOX (container);
-  gboolean hexpand = FALSE, hfill = FALSE, vexpand = FALSE, vfill = FALSE;
+  gboolean hexpand = FALSE, hfill = FALSE, vexpand = FALSE, vfill = FALSE, wrapped = FALSE;
   
   if (arg_id != CHILD_ARG_POSITION)
-    gtk_wrap_box_query_child_packing (wbox, child, &hexpand, &hfill, &vexpand, &vfill);
+    gtk_wrap_box_query_child_packing (wbox, child, &hexpand, &hfill, &vexpand, &vfill, &wrapped);
   
   switch (arg_id)
     {
@@ -287,22 +290,32 @@ gtk_wrap_box_set_child_arg (GtkContainer *container,
     case CHILD_ARG_HEXPAND:
       gtk_wrap_box_set_child_packing (wbox, child,
 				      GTK_VALUE_BOOL (*arg), hfill,
-				      vexpand, vfill);
+				      vexpand, vfill,
+				      wrapped);
       break;
     case CHILD_ARG_HFILL:
       gtk_wrap_box_set_child_packing (wbox, child,
 				      hexpand, GTK_VALUE_BOOL (*arg),
-				      vexpand, vfill);
+				      vexpand, vfill,
+				      wrapped);
       break;
     case CHILD_ARG_VEXPAND:
       gtk_wrap_box_set_child_packing (wbox, child,
 				      hexpand, hfill,
-				      GTK_VALUE_BOOL (*arg), vfill);
+				      GTK_VALUE_BOOL (*arg), vfill,
+				      wrapped);
       break;
     case CHILD_ARG_VFILL:
       gtk_wrap_box_set_child_packing (wbox, child,
 				      hexpand, hfill,
-				      vexpand, GTK_VALUE_BOOL (*arg));
+				      vexpand, GTK_VALUE_BOOL (*arg),
+				      wrapped);
+      break;
+    case CHILD_ARG_WRAPPED:
+      gtk_wrap_box_set_child_packing (wbox, child,
+				      hexpand, hfill,
+				      vexpand, vfill,
+				      GTK_VALUE_BOOL (*arg));
       break;
     default:
       break;
@@ -316,10 +329,10 @@ gtk_wrap_box_get_child_arg (GtkContainer *container,
 			    guint         arg_id)
 {
   GtkWrapBox *wbox = GTK_WRAP_BOX (container);
-  gboolean hexpand = FALSE, hfill = FALSE, vexpand = FALSE, vfill = FALSE;
+  gboolean hexpand = FALSE, hfill = FALSE, vexpand = FALSE, vfill = FALSE, wrapped = FALSE;
   
   if (arg_id != CHILD_ARG_POSITION)
-    gtk_wrap_box_query_child_packing (wbox, child, &hexpand, &hfill, &vexpand, &vfill);
+    gtk_wrap_box_query_child_packing (wbox, child, &hexpand, &hfill, &vexpand, &vfill, &wrapped);
   
   switch (arg_id)
     {
@@ -347,6 +360,9 @@ gtk_wrap_box_get_child_arg (GtkContainer *container,
     case CHILD_ARG_VFILL:
       GTK_VALUE_BOOL (*arg) = vfill;
       break;
+    case CHILD_ARG_WRAPPED:
+      GTK_VALUE_BOOL (*arg) = wrapped;
+      break;
     default:
       arg->type = GTK_TYPE_INVALID;
       break;
@@ -449,6 +465,22 @@ gtk_wrap_box_pack (GtkWrapBox *wbox,
 		   gboolean    hfill,
 		   gboolean    vexpand,
 		   gboolean    vfill)
+{
+  g_return_if_fail (GTK_IS_WRAP_BOX (wbox));
+  g_return_if_fail (GTK_IS_WIDGET (child));
+  g_return_if_fail (child->parent == NULL);
+
+  gtk_wrap_box_pack_wrapped (wbox, child, hexpand, hfill, vexpand, vfill, FALSE);
+}
+
+void
+gtk_wrap_box_pack_wrapped (GtkWrapBox *wbox,
+			   GtkWidget  *child,
+			   gboolean    hexpand,
+			   gboolean    hfill,
+			   gboolean    vexpand,
+			   gboolean    vfill,
+			   gboolean    wrapped)
 {
   GtkWrapBoxChild *child_info;
   
@@ -462,6 +494,7 @@ gtk_wrap_box_pack (GtkWrapBox *wbox,
   child_info->hfill = hfill ? TRUE : FALSE;
   child_info->vexpand = vexpand ? TRUE : FALSE;
   child_info->vfill = vfill ? TRUE : FALSE;
+  child_info->wrapped = wrapped ? TRUE : FALSE;
   child_info->next = NULL;
   if (wbox->children)
     {
@@ -546,7 +579,8 @@ gtk_wrap_box_query_child_packing (GtkWrapBox *wbox,
 				  gboolean   *hexpand,
 				  gboolean   *hfill,
 				  gboolean   *vexpand,
-				  gboolean   *vfill)
+				  gboolean   *vfill,
+				  gboolean   *wrapped)
 {
   GtkWrapBoxChild *child_info;
   
@@ -567,6 +601,8 @@ gtk_wrap_box_query_child_packing (GtkWrapBox *wbox,
 	*vexpand = child_info->vexpand;
       if (vfill)
 	*vfill = child_info->vfill;
+      if (wrapped)
+	*wrapped = child_info->wrapped;
     }
 }
 
@@ -576,7 +612,8 @@ gtk_wrap_box_set_child_packing (GtkWrapBox *wbox,
 				gboolean    hexpand,
 				gboolean    hfill,
 				gboolean    vexpand,
-				gboolean    vfill)
+				gboolean    vfill,
+				gboolean    wrapped)
 {
   GtkWrapBoxChild *child_info;
   
@@ -587,19 +624,22 @@ gtk_wrap_box_set_child_packing (GtkWrapBox *wbox,
   hfill = hfill != FALSE;
   vexpand = vexpand != FALSE;
   vfill = vfill != FALSE;
-  
+  wrapped = wrapped != FALSE;
+
   for (child_info = wbox->children; child_info; child_info = child_info->next)
     if (child_info->widget == child)
       break;
   
   if (child_info &&
       (child_info->hexpand != hexpand || child_info->vexpand != vexpand ||
-       child_info->hfill != hfill || child_info->vfill != vfill))
+       child_info->hfill != hfill || child_info->vfill != vfill ||
+       child_info->wrapped != wrapped))
     {
       child_info->hexpand = hexpand;
       child_info->hfill = hfill;
       child_info->vexpand = vexpand;
       child_info->vfill = vfill;
+      child_info->wrapped = wrapped;
       
       if (GTK_WIDGET_VISIBLE (child) && GTK_WIDGET_VISIBLE (wbox))
 	gtk_widget_queue_resize (child);
@@ -1094,7 +1134,7 @@ gtk_clue_hunter_clist_click (GtkClueHunter  *clue_hunter,
       g_free (string);
 
       gtk_clue_hunter_popdown (clue_hunter);
-      gtk_widget_activate (clue_hunter->entry);
+      /* gtk_widget_activate (clue_hunter->entry); */
     }
 
   return handled;
@@ -1221,8 +1261,8 @@ gtk_clue_hunter_entry_key_press (GtkClueHunter *clue_hunter,
 {
   gboolean handled = FALSE;
   
-  if (!(event->state == GDK_CONTROL_MASK || event->state == GDK_MOD1_MASK) &&
-      (event->keyval == GDK_Tab || event->keyval == GDK_ISO_Left_Tab))
+  if ((event->keyval == GDK_Tab || event->keyval == GDK_ISO_Left_Tab) &&
+      !(event->state & (GDK_SHIFT_MASK | GDK_CONTROL_MASK | GDK_MOD1_MASK)))
     {
       handled = TRUE;
       
@@ -1531,11 +1571,11 @@ gtk_clue_hunter_do_popup (GtkClueHunter *clue_hunter)
   
   widget = GTK_WIDGET (clue_hunter);
   
+  gtk_widget_grab_focus (clue_hunter->entry);
+
   if (!clue_hunter->cstring)
     clue_hunter->cstring = g_strdup ("");
   
-  gtk_widget_grab_focus (clue_hunter->entry);
-
   gtk_clist_columns_autosize (GTK_CLIST (clue_hunter->clist));
   gtk_widget_size_request (clue_hunter->clist, NULL);
   gtk_widget_set_usize (clue_hunter->clist,
@@ -1770,7 +1810,7 @@ gtk_clue_hunter_event (GtkWidget *widget,
 		gtk_clue_hunter_add_history (clue_hunter, string);
 	    }
 	  gtk_clue_hunter_popdown (clue_hunter);
-	  gtk_widget_activate (clue_hunter->entry);
+	  /* gtk_widget_activate (clue_hunter->entry); */
 	}
       else
 	handled = gtk_widget_event (clue_hunter->entry, event);
@@ -2156,7 +2196,8 @@ reverse_list_row_children (GtkWrapBox       *wbox,
 	  if (GTK_WIDGET_VISIBLE (child->widget))
 	    {
 	      get_gtkhwrapbox_child_requisition (wbox, child->widget, &child_requisition);
-	      if (width + wbox->hspacing + child_requisition.width > row_width)
+	      if (width + wbox->hspacing + child_requisition.width > row_width ||
+		  child->wrapped)
 		break;
 	      width += wbox->hspacing + child_requisition.width;
 	      *max_child_size = MAX (*max_child_size, child_requisition.height);
@@ -2180,7 +2221,8 @@ layout_row (GtkWrapBox    *wbox,
 	    gboolean       vexpand)
 {
   GSList *slist;
-  guint n_children = 0, n_expand_children = 0, have_expand_children = 0, total_width = 0;
+  guint n_children = 0, n_expand_children = 0, have_expand_children = 0;
+  gint total_width = 0;
   gfloat x, width, extra;
   GtkAllocation child_allocation;
   
@@ -2211,7 +2253,7 @@ layout_row (GtkWrapBox    *wbox,
       width /= ((gdouble) children_per_line);
       extra = 0;
     }
-  else if (have_expand_children)
+  else if (have_expand_children && wbox->justify != GTK_JUSTIFY_FILL)
     {
       width = extra;
       extra /= ((gdouble) n_expand_children);
@@ -2452,7 +2494,7 @@ layout_rows (GtkWrapBox    *wbox,
 		      line->children,
 		      children_per_line,
 		      line->expand);
-	  
+
 	  g_slist_free (line->children);
 	  g_free (line);
 	  line = next_line;
@@ -2466,7 +2508,7 @@ gtk_hwrap_box_size_allocate (GtkWidget     *widget,
 {
   GtkWrapBox *wbox = GTK_WRAP_BOX (widget);
   GtkAllocation area;
-  guint border = GTK_CONTAINER (wbox)->border_width; /*<h2v-skip>*/
+  gint border = GTK_CONTAINER (wbox)->border_width; /*<h2v-skip>*/
   
   widget->allocation = *allocation;
   area.x = allocation->x + border;
@@ -2809,7 +2851,8 @@ reverse_list_col_children (GtkWrapBox       *wbox,
 	  if (GTK_WIDGET_VISIBLE (child->widget))
 	    {
 	      get_gtkvwrapbox_child_requisition (wbox, child->widget, &child_requisition);
-	      if (height + wbox->vspacing + child_requisition.height > col_height)
+	      if (height + wbox->vspacing + child_requisition.height > col_height ||
+		  child->wrapped)
 		break;
 	      height += wbox->vspacing + child_requisition.height;
 	      *max_child_size = MAX (*max_child_size, child_requisition.width);
@@ -2833,7 +2876,8 @@ layout_col (GtkWrapBox    *wbox,
 	    gboolean       hexpand)
 {
   GSList *slist;
-  guint n_children = 0, n_expand_children = 0, have_expand_children = 0, total_height = 0;
+  guint n_children = 0, n_expand_children = 0, have_expand_children = 0;
+  gint total_height = 0;
   gfloat y, height, extra;
   GtkAllocation child_allocation;
   
@@ -2864,7 +2908,7 @@ layout_col (GtkWrapBox    *wbox,
       height /= ((gdouble) children_per_line);
       extra = 0;
     }
-  else if (have_expand_children)
+  else if (have_expand_children && wbox->justify != GTK_JUSTIFY_FILL)
     {
       height = extra;
       extra /= ((gdouble) n_expand_children);
@@ -3105,7 +3149,7 @@ layout_cols (GtkWrapBox    *wbox,
 		      line->children,
 		      children_per_line,
 		      line->expand);
-	  
+
 	  g_slist_free (line->children);
 	  g_free (line);
 	  line = next_line;
@@ -3119,7 +3163,7 @@ gtk_vwrap_box_size_allocate (GtkWidget     *widget,
 {
   GtkWrapBox *wbox = GTK_WRAP_BOX (widget);
   GtkAllocation area;
-  guint border = GTK_CONTAINER (wbox)->border_width; /*<h2v-skip>*/
+  gint border = GTK_CONTAINER (wbox)->border_width; /*<h2v-skip>*/
   
   widget->allocation = *allocation;
   area.y = allocation->y + border;
@@ -3136,99 +3180,3 @@ gtk_vwrap_box_size_allocate (GtkWidget     *widget,
   
   layout_cols (wbox, &area);
 }
-void
-gtk_file_selection_heal (GtkFileSelection *fs)
-{
-  GtkWidget *main_vbox;
-  GtkWidget *hbox;
-  GtkWidget *any;
-
-  g_return_if_fail (fs != NULL);
-  g_return_if_fail (GTK_IS_FILE_SELECTION (fs));
-
-  /* button placement
-   */
-  gtk_container_set_border_width (GTK_CONTAINER (fs), 0);
-  gtk_file_selection_hide_fileop_buttons (fs);
-  gtk_widget_ref (fs->main_vbox);
-  gtk_container_remove (GTK_CONTAINER (fs), fs->main_vbox);
-  gtk_box_set_spacing (GTK_BOX (fs->main_vbox), 0);
-  gtk_container_set_border_width (GTK_CONTAINER (fs->main_vbox), 5);
-  main_vbox =
-    gtk_widget_new (GTK_TYPE_VBOX,
-		    "homogeneous", FALSE,
-		    "spacing", 0,
-		    "border_width", 0,
-		    "parent", fs,
-		    "visible", TRUE,
-		    NULL);
-  gtk_box_pack_start (GTK_BOX (main_vbox), fs->main_vbox, TRUE, TRUE, 0);
-  gtk_widget_unref (fs->main_vbox);
-  gtk_widget_hide (fs->ok_button->parent);
-  hbox =
-    gtk_widget_new (GTK_TYPE_HBOX,
-		    "homogeneous", TRUE,
-		    "spacing", 0,
-		    "border_width", 5,
-		    "visible", TRUE,
-		    NULL);
-  gtk_box_pack_end (GTK_BOX (main_vbox), hbox, FALSE, TRUE, 0);
-  gtk_widget_reparent (fs->ok_button, hbox);
-  gtk_widget_reparent (fs->cancel_button, hbox);
-  gtk_widget_grab_default (fs->ok_button);
-  gtk_label_set_text (GTK_LABEL (GTK_BIN (fs->ok_button)->child), "Ok");
-  gtk_label_set_text (GTK_LABEL (GTK_BIN (fs->cancel_button)->child), "Cancel");
-
-  /* heal the action_area packing so we can customize children
-   */
-  gtk_box_set_child_packing (GTK_BOX (fs->action_area->parent),
-			     fs->action_area,
-			     FALSE, TRUE,
-			     5, GTK_PACK_START);
-
-  any =
-    gtk_widget_new (gtk_hseparator_get_type (),
-		    "GtkWidget::visible", TRUE,
-		    NULL);
-  gtk_box_pack_end (GTK_BOX (main_vbox), any, FALSE, TRUE, 0);
-  gtk_widget_grab_focus (fs->selection_entry);
-}
-
-static gint
-idle_shower (GtkWidget **widget_p)
-{
-  GDK_THREADS_ENTER ();
-  
-  if (GTK_IS_WIDGET (*widget_p) && !GTK_OBJECT_DESTROYED (*widget_p))
-    {
-      gtk_signal_disconnect_by_func (GTK_OBJECT (*widget_p),
-				     GTK_SIGNAL_FUNC (gtk_widget_destroyed),
-				     widget_p);
-      gtk_widget_show (*widget_p);
-    }
-
-  g_free (widget_p);
-
-  GDK_THREADS_LEAVE ();
-
-  return FALSE;
-}
-
-void
-gtk_idle_show_widget (GtkWidget *widget)
-{
-  GtkWidget **widget_p;
-
-  g_return_if_fail (GTK_IS_WIDGET (widget));
-  if (GTK_OBJECT_DESTROYED (widget))
-    return;
-
-  widget_p = g_new (GtkWidget*, 1);
-  *widget_p = widget;
-  gtk_signal_connect (GTK_OBJECT (widget),
-		      "destroy",
-		      GTK_SIGNAL_FUNC (gtk_widget_destroyed),
-		      widget_p);
-  gtk_idle_add_priority (G_PRIORITY_LOW, (GtkFunction) idle_shower, widget_p);
-}
-
