@@ -49,11 +49,12 @@
 #include        <bse/bseitem.h>
 #include        <bse/bsemagic.h>
 #include        <bse/bsemain.h>
-#include        <bse/bsemixer.h>
+#include        <bse/bsehunkmixer.h>
 #include        <bse/bseobject.h>
 #include        <bse/bseparam.h>
 #include        <bse/bseparasite.h>
 #include        <bse/bsepattern.h>
+#include        <bse/bsepatterngroup.h>
 #include        <bse/bsepcmdevice.h>
 #include        <bse/bseplugin.h>
 #include        <bse/bseprocedure.h>
