@@ -87,15 +87,6 @@ static BseEnumValue bse_loop_type_values[] = {
   { 0, NULL, NULL }
 };
 
-/* BseInstrumentType
- */
-static BseEnumValue bse_instrument_type_values[] = {
-  { BSE_INSTRUMENT_NONE, "BSE_INSTRUMENT_NONE", "none" },
-  { BSE_INSTRUMENT_SAMPLE, "BSE_INSTRUMENT_SAMPLE", "sample" },
-  { BSE_INSTRUMENT_MIDI, "BSE_INSTRUMENT_MIDI", "midi" },
-  { 0, NULL, NULL }
-};
-
 /* BseErrorType
  */
 static BseEnumValue bse_error_type_values[] = {
