@@ -17,6 +17,8 @@
  */
 #include	"bseenums.h"
 
+#include        <bse/bseexports.h>
+#include	<errno.h>
 
 /* --- prototypes --- */
 extern void	bse_type_register_enums		(void);
@@ -88,6 +90,57 @@ bse_type_register_enums (void)
     }
 }
 
+void
+bse_enum_complete_info (const BseExportSpec *spec,
+			BseTypeInfo         *info)
+{
+  const BseExportEnum *espec = &spec->s_enum;
+  
+  if (espec->parent_type == BSE_TYPE_ENUM)
+    {
+      info->class_size = sizeof (BseEnumClass);
+      info->class_init = (BseClassInitFunc) bse_enum_class_init;
+    }
+  else if (espec->parent_type == BSE_TYPE_FLAGS)
+    {
+      info->class_size = sizeof (BseFlagsClass);
+      info->class_init = (BseClassInitFunc) bse_flags_class_init;
+    }
+  else
+    g_assert_not_reached ();
+  
+  info->class_data = espec->values;
+}
+
+const gchar*
+bse_enum_type_register (const gchar *name,
+			BseType      parent_type,
+			BsePlugin   *plugin,
+			BseType     *ret_type)
+{
+  BseType type;
+
+  g_return_val_if_fail (ret_type != NULL, bse_error_blurb (BSE_ERROR_INTERNAL));
+  *ret_type = 0;
+  g_return_val_if_fail (name != NULL, bse_error_blurb (BSE_ERROR_INTERNAL));
+  g_return_val_if_fail (parent_type != 0, bse_error_blurb (BSE_ERROR_INTERNAL));
+  g_return_val_if_fail (plugin != NULL, bse_error_blurb (BSE_ERROR_INTERNAL));
+
+  type = bse_type_from_name (name);
+  if (type)
+    return "Enum Type already registered";
+  if (parent_type != BSE_TYPE_ENUM && parent_type != parent_type)
+    return "Parent type neither enum nor flags";
+
+  type = bse_type_register_dynamic (parent_type,
+				    name,
+				    NULL,
+				    plugin);
+  *ret_type = type;
+
+  return NULL;
+}
+
 static void
 bse_enum_class_init (BseEnumClass *class,
 		     gpointer	   class_data)
@@ -280,11 +333,12 @@ bse_error_blurb (BseErrorType error_value)
   switch (error_value)
     {
     case BSE_ERROR_NONE:			return "Everything went well";
-    case BSE_ERROR_IGNORE:			return "Something went wrong...";
+    case BSE_ERROR_IGNORE:			return "Temporary headache...";
     case BSE_ERROR_UNKNOWN:			return "Unknown error";
-    case BSE_ERROR_INTERNAL:			return "Internal error";
+    case BSE_ERROR_INTERNAL:			return "Internal error (please report)";
     case BSE_ERROR_UNIMPLEMENTED:		return "Functionality not imlemented";
-    case BSE_ERROR_FILE_IO:			return "File I/O error";
+    case BSE_ERROR_IO:				return "Device/file I/O error";
+    case BSE_ERROR_PERMS:			return "Insufficient permissions";
     case BSE_ERROR_FILE_EXISTS:			return "File exists";
     case BSE_ERROR_FILE_NOT_FOUND:		return "File not found";
     case BSE_ERROR_FILE_TOO_SHORT:		return "File too short";
@@ -296,10 +350,8 @@ bse_error_blurb (BseErrorType error_value)
     case BSE_ERROR_SUB_HEADER_CORRUPT:		return "Sub-header corrupt";
     case BSE_ERROR_DATA_CORRUPT:		return "Data corrupt";
     case BSE_ERROR_BINARY_DATA_CORRUPT:		return "Binary data corrupt";
-    case BSE_ERROR_DEVICE_PERMS:		return "Device permissions insufficient";
     case BSE_ERROR_DEVICE_ASYNC:		return "Device not async capable";
     case BSE_ERROR_DEVICE_BUSY:			return "Device busy";
-    case BSE_ERROR_DEVICE_IO:			return "Device I/O error";
     case BSE_ERROR_DEVICE_GET_CAPS:		return "Failed to query device capabilities";
     case BSE_ERROR_DEVICE_CAPS_MISMATCH:	return "Device capabilities not sufficient";
     case BSE_ERROR_DEVICE_SET_CAPS:		return "Failed to set device capabilities";
@@ -321,3 +373,23 @@ bse_error_blurb (BseErrorType error_value)
   ev = bse_enum_get_value (bse_error_class, error_value);
   return ev ? ev->value_nick : NULL;
 }
+
+BseErrorType
+bse_error_from_errno (gint            v_errno,
+		      BseErrorType    fallback)
+{
+  switch (v_errno)
+    {
+    case EBUSY:		return BSE_ERROR_DEVICE_BUSY;
+    case EISDIR:
+    case EACCES:
+    case EPERM:
+    case EROFS:		return BSE_ERROR_DEVICE_PERMS;
+    case ELOOP:
+    case ENOENT:	return BSE_ERROR_FILE_NOT_FOUND;
+    case EEXIST:	return BSE_ERROR_FILE_EXISTS;
+    case EIO:		return BSE_ERROR_FILE_IO;
+    case EBADF:		return BSE_ERROR_INTERNAL;
+    default:		return fallback;
+    }      
+}
