@@ -15,8 +15,6 @@ BseType BSE_TYPE_ID (BseEndianType) = 0;
 
 BseType BSE_TYPE_ID (BseLoopType) = 0;
 
-BseType BSE_TYPE_ID (BseInstrumentType) = 0;
-
 BseType BSE_TYPE_ID (BseErrorType) = 0;
 
 BseType BSE_TYPE_ID (BseMagicFlags) = 0;
@@ -43,6 +41,10 @@ BseType BSE_TYPE_ID (GMatchType) = 0;
 BSE_BUILTIN_PROTO (BseBinData);
 BseType bse_type_builtin_id_BseBinData = 0;
 
+/* --- bsecapture.c --- */
+BSE_BUILTIN_PROTO (BseCapture);
+BseType bse_type_builtin_id_BseCapture = 0;
+
 /* --- bsecontainer.c --- */
 BSE_BUILTIN_PROTO (BseContainer);
 BseType bse_type_builtin_id_BseContainer = 0;
@@ -95,6 +97,10 @@ BseType bse_type_builtin_id_BseItem = 0;
 BSE_BUILTIN_PROTO (BsePattern);
 BseType bse_type_builtin_id_BsePattern = 0;
 
+/* --- bsepatterngroup.c --- */
+BSE_BUILTIN_PROTO (BsePatternGroup);
+BseType bse_type_builtin_id_BsePatternGroup = 0;
+
 /* --- bsepcmdevice.c --- */
 BSE_BUILTIN_PROTO (BsePcmDevice);
 BseType bse_type_builtin_id_BsePcmDevice = 0;
@@ -115,6 +121,10 @@ BseType bse_type_builtin_id_BseProject = 0;
 BSE_BUILTIN_PROTO (BseSample);
 BseType bse_type_builtin_id_BseSample = 0;
 
+/* --- bsesinstrument.c --- */
+BSE_BUILTIN_PROTO (BseSInstrument);
+BseType bse_type_builtin_id_BseSInstrument = 0;
+
 /* --- bsesnet.c --- */
 BSE_BUILTIN_PROTO (BseSNet);
 BseType bse_type_builtin_id_BseSNet = 0;
