@@ -24,8 +24,6 @@ extern BseType BSE_TYPE_ID (BseEffectType);
 extern BseType BSE_TYPE_ID (BseEndianType);
 #define BSE_TYPE_LOOP_TYPE	    (BSE_TYPE_ID (BseLoopType))
 extern BseType BSE_TYPE_ID (BseLoopType);
-#define BSE_TYPE_INSTRUMENT_TYPE	    (BSE_TYPE_ID (BseInstrumentType))
-extern BseType BSE_TYPE_ID (BseInstrumentType);
 #define BSE_TYPE_ERROR_TYPE	    (BSE_TYPE_ID (BseErrorType))
 extern BseType BSE_TYPE_ID (BseErrorType);
 
@@ -62,6 +60,9 @@ extern BseType BSE_TYPE_ID (GMatchType);
 /* --- bsebindata.c --- */
 extern BseType bse_type_builtin_id_BseBinData;
 
+/* --- bsecapture.c --- */
+extern BseType bse_type_builtin_id_BseCapture;
+
 /* --- bsecontainer.c --- */
 extern BseType bse_type_builtin_id_BseContainer;
 
@@ -101,6 +102,9 @@ extern BseType bse_type_builtin_id_BseItem;
 /* --- bsepattern.c --- */
 extern BseType bse_type_builtin_id_BsePattern;
 
+/* --- bsepatterngroup.c --- */
+extern BseType bse_type_builtin_id_BsePatternGroup;
+
 /* --- bsepcmdevice.c --- */
 extern BseType bse_type_builtin_id_BsePcmDevice;
 
@@ -116,6 +120,9 @@ extern BseType bse_type_builtin_id_BseProject;
 /* --- bsesample.c --- */
 extern BseType bse_type_builtin_id_BseSample;
 
+/* --- bsesinstrument.c --- */
+extern BseType bse_type_builtin_id_BseSInstrument;
+
 /* --- bsesnet.c --- */
 extern BseType bse_type_builtin_id_BseSNet;
 
