@@ -18,7 +18,7 @@
 #include "bseheart.h"
 
 #include "bsechunk.h"
-
+#include "bsehunkmixer.h"
 
 /* --- parameters --- */
 enum
@@ -34,9 +34,11 @@ static void        bse_heart_class_init    (BseHeartClass *class);
 static void        bse_heart_shutdown      (BseObject     *object);
 static void        bse_heart_destroy       (BseObject     *object);
 static void        bse_heart_set_param     (BseHeart	  *heart,
-					    BseParam      *param);
+					    BseParam      *param,
+					    guint          param_id);
 static void        bse_heart_get_param     (BseHeart	  *heart,
-					    BseParam      *param);
+					    BseParam      *param,
+					    guint          param_id);
 static gboolean	   bse_heart_prepare	   (gpointer       source_data,
 					    GTimeVal      *current_time,
 					    gint          *timeout,
@@ -74,14 +76,13 @@ BSE_BUILTIN_TYPE (BseHeart)
     NULL /* class_data */,
 
     sizeof (BseHeart),
-    16 /* n_preallocs */,
+    0 /* n_preallocs */,
     (BseObjectInitFunc) bse_heart_init,
   };
 
   return bse_type_register_static (BSE_TYPE_OBJECT,
 				   "BseHeart",
-				   "Project container for administration "
-				   "of Source networks",
+				   "BSE Heart - fundamental audio router",
 				   &heart_info);
 }
 
@@ -175,38 +176,32 @@ bse_heart_destroy (BseObject *object)
 
 static void
 bse_heart_set_param (BseHeart *heart,
-		     BseParam *param)
+		     BseParam *param,
+		     guint     param_id)
 {
-  switch (param->pspec->any.param_id)
+  switch (param_id)
     {
     case PARAM_LATENCY:
       heart->latency = param->value.v_uint;
       break;
     default:
-      g_warning ("%s(\"%s\"): invalid attempt to set parameter \"%s\" of type `%s'",
-		 BSE_OBJECT_TYPE_NAME (heart),
-		 BSE_OBJECT_NAME (heart),
-		 param->pspec->any.name,
-		 bse_type_name (param->pspec->type));
+      BSE_UNHANDLED_PARAM_ID (heart, param, param_id);
       break;
     }
 }
 
 static void
 bse_heart_get_param (BseHeart *heart,
-		     BseParam *param)
+                     BseParam *param,
+		     guint     param_id)
 {
-  switch (param->pspec->any.param_id)
+  switch (param_id)
     {
     case PARAM_LATENCY:
       param->value.v_uint = heart->latency;
       break;
     default:
-      g_warning ("%s(\"%s\"): invalid attempt to get parameter \"%s\" of type `%s'",
-		 BSE_OBJECT_TYPE_NAME (heart),
-		 BSE_OBJECT_NAME (heart),
-		 param->pspec->any.name,
-		 bse_type_name (param->pspec->type));
+      BSE_UNHANDLED_PARAM_ID (heart, param, param_id);
       break;
     }
 }
@@ -543,8 +538,8 @@ bse_heart_source_add_odevice (BseSource    *source,
   g_return_if_fail (!BSE_SOURCE_OATTACHED (source));
   g_return_if_fail (BSE_IS_PCM_DEVICE (odev));
   g_return_if_fail (BSE_DEVICE_REGISTERED (odev));
-  g_return_if_fail (BSE_SOURCE_N_OCHANNELS (source) == 1);
-  g_return_if_fail (BSE_SOURCE_OCHANNEL_DEF (source, 1)->n_tracks <= 2);
+  g_return_if_fail (BSE_SOURCE_N_OCHANNELS (source) >= BSE_DFL_OCHANNEL_ID);
+  g_return_if_fail (BSE_SOURCE_OCHANNEL_DEF (source, BSE_DFL_OCHANNEL_ID)->n_tracks <= 2);
 
   heart = bse_heart_get_global (FALSE);
   
@@ -667,7 +662,7 @@ bse_heart_queue_device (BseHeart       *heart,
 
   heart->device_open_list = g_slist_prepend (heart->device_open_list, hdevice);
   if (!heart->device_open_handler_id)
-    heart->device_open_handler_id = g_idle_add_full (BSE_HEART_PRIORITY - 1,
+    heart->device_open_handler_id = g_idle_add_full (BSE_DEVICE_PRIORITY,
 						     device_open_handler,
 						     heart,
 						     NULL);
@@ -683,8 +678,8 @@ bse_heart_beat (BseHeart *heart)
   /* no, sources may NOT be removed or added during cycling
    */
 
-  /* turbo-cycle recently added sources */
-  for (i = 0; i < heart->n_sources; i++) /* FIXME */
+  /* turbo-cycle recently added sources */ /* FIXME */
+  for (i = 0; i < heart->n_sources; i++)
     {
       BseSource *source = heart->sources[i];
 
@@ -716,7 +711,7 @@ bse_heart_collect_chunks (BseHeart       *heart,
 
   for (i = 0; i < hdevice->n_osources; i++)
     slist = g_slist_prepend (slist, bse_source_ref_chunk (hdevice->osources[i],
-							  1,
+							  BSE_DFL_OCHANNEL_ID,
 							  bse_heart_beat_index));
 
   return slist;
@@ -727,7 +722,7 @@ bse_heart_mix_chunks (BseHeart *heart,
 		      GSList   *chunk_list, /* auto frees list and unrefs chunks */
 		      guint     n_tracks)
 {
-  BseMixValue *mb, *mbe, *mv;
+  BseMixValue *mix_buffer, *bound, *mv;
   BseChunk *chunk;
   BseSampleValue *sv;
   GSList *slist;
@@ -735,9 +730,9 @@ bse_heart_mix_chunks (BseHeart *heart,
 
   g_return_val_if_fail (BSE_IS_HEART (heart), NULL);
 
-  mb = heart->mix_buffer;
-  memset (mb, 0, sizeof (BseMixValue) * n_tracks * track_length);
-  mbe = mb + track_length * n_tracks;
+  mix_buffer = heart->mix_buffer;
+  memset (mix_buffer, 0, sizeof (BseMixValue) * n_tracks * track_length);
+  bound = mix_buffer + track_length * n_tracks;
 
   for (slist = chunk_list; slist; slist = slist->next)
     {
@@ -745,10 +740,10 @@ bse_heart_mix_chunks (BseHeart *heart,
 
       sv = chunk->hunk;
       if (n_tracks == chunk->n_tracks)
-	for (mv = mb; mv < mbe; mv++)
+	for (mv = mix_buffer; mv < bound; mv++)
 	  *mv += *(sv++);
       else if (n_tracks == chunk->n_tracks * 2)
-	for (mv = mb; mv < mbe; mv++)
+	for (mv = mix_buffer; mv < bound; mv++)
 	  {
 	    register BseMixValue v = *(sv++);
 
@@ -756,7 +751,7 @@ bse_heart_mix_chunks (BseHeart *heart,
 	    *mv += v >> 2;
 	  }
       else if (n_tracks * 2 == chunk->n_tracks)
-	for (mv = mb; mv < mbe; mv++)
+	for (mv = mix_buffer; mv < bound; mv++)
 	  {
 	    *(mv++) += *sv;
 	    *mv += *(sv++);
@@ -768,17 +763,7 @@ bse_heart_mix_chunks (BseHeart *heart,
   g_slist_free (chunk_list);
 
   chunk = bse_chunk_new (n_tracks);
-  sv = chunk->hunk;
-  for (mv = mb; mv < mbe; mv++)
-    {
-      register BseMixValue v = *mv;
-
-      if (v > 32767)
-	v = 32767;
-      else if (v < -32768)
-	v = -32768;
-      *(sv++) = v;
-    }
+  bse_hunk_clip_from_mix_buffer (n_tracks, chunk->hunk, 1.0, mix_buffer);
   chunk->hunk_filled = TRUE;
 
   return chunk;
