@@ -1,5 +1,5 @@
 /* BSE - Bedevilled Sound Engine
- * Copyright (C) 1997, 1998, 1999 Olaf Hoehmann and Tim Janik
+ * Copyright (C) 1997, 1998, 1999, 2000 Olaf Hoehmann and Tim Janik
  *
  * This library is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,12 +16,15 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
  */
 #include	"bseinstrument.h"
+
+#include	"bsesinstrument.h"
 #include	"bsesample.h"
 #include	"bseglobals.h"
 
 
 enum {
   PARAM_0,
+  PARAM_SYNTH,
   PARAM_SAMPLE,
   PARAM_INTERPOLATION,
   PARAM_POLYPHONY,
@@ -79,9 +82,11 @@ static void	bse_instrument_class_init	(BseInstrumentClass	*class);
 static void	bse_instrument_init		(BseInstrument		*instrument);
 static void	bse_instrument_do_shutdown	(BseObject		*object);
 static void	bse_instrument_set_param	(BseInstrument		*instrument,
-						 BseParam		*param);
+						 BseParam		*param,
+						 guint                   param_id);
 static void	bse_instrument_get_param	(BseInstrument		*instrument,
-						 BseParam		*param);
+						 BseParam		*param,
+						 guint                   param_id);
 static void	bse_instrument_unlocked		(BseObject		*object);
 
 
@@ -102,7 +107,7 @@ BSE_BUILTIN_TYPE (BseInstrument)
     NULL /* class_data */,
     
     sizeof (BseInstrument),
-    8 /* n_preallocs */,
+    BSE_PREALLOC_N_INSTRUMENTS /* n_preallocs */,
     (BseObjectInitFunc) bse_instrument_init,
   };
   
@@ -127,17 +132,22 @@ bse_instrument_class_init (BseInstrumentClass *class)
   object_class->unlocked = bse_instrument_unlocked;
   object_class->shutdown = bse_instrument_do_shutdown;
   
-  bse_object_class_add_param (object_class, NULL,
+  bse_object_class_add_param (object_class, "Synthesis Input",
+			      PARAM_SYNTH,
+			      bse_param_spec_item ("sinstrument", "Synth", NULL,
+						   BSE_TYPE_SINSTRUMENT,
+						   (BSE_PARAM_DEFAULT) & ~BSE_PARAM_SERVE_GUI));
+  bse_object_class_add_param (object_class, "Sample Input",
 			      PARAM_SAMPLE,
 			      bse_param_spec_item ("sample", "Sample", NULL,
 						   BSE_TYPE_SAMPLE,
 						   BSE_PARAM_DEFAULT));
-  bse_object_class_add_param (object_class, NULL,
+  bse_object_class_add_param (object_class, "Sample Input",
 			      PARAM_INTERPOLATION,
 			      bse_param_spec_bool ("interpolation", "Use interpolation?", NULL /* FIXME */,
 						   TRUE,
 						   BSE_PARAM_DEFAULT));
-  bse_object_class_add_param (object_class, NULL,
+  bse_object_class_add_param (object_class, "Sample Input",
 			      PARAM_POLYPHONY,
 			      bse_param_spec_bool ("polyphony", "Polyphony instrument?", NULL,
 						   FALSE,
@@ -162,7 +172,7 @@ bse_instrument_class_init (BseInstrumentClass *class)
 			      bse_param_spec_uint ("volume_perc", "Volume [%]", NULL,
 						   0, bse_dB_to_factor (BSE_MAX_VOLUME_dB) * 100,
 						   1,
-						   bse_dB_to_factor (BSE_DFL_SONG_VOLUME_dB) * 100,
+						   bse_dB_to_factor (BSE_DFL_INSTRUMENT_VOLUME_dB) * 100,
 						   BSE_PARAM_GUI |
 						   BSE_PARAM_HINT_DIAL));
   bse_object_class_add_param (object_class, "Adjustments",
@@ -267,11 +277,7 @@ static void
 bse_instrument_init (BseInstrument *instrument)
 {
   instrument->type = BSE_INSTRUMENT_NONE;
-  instrument->sample = NULL;
-  instrument->locked_sample = NULL;
-  
-  instrument->interpolation = TRUE;
-  instrument->polyphony = FALSE;
+
   instrument->volume_factor = bse_dB_to_factor (BSE_DFL_INSTRUMENT_VOLUME_dB);
   instrument->balance = BSE_DFL_INSTRUMENT_BALANCE;
   instrument->transpose = BSE_DFL_INSTRUMENT_TRANSPOSE;
@@ -292,16 +298,7 @@ bse_instrument_do_shutdown (BseObject *object)
 {
   BseInstrument *instrument = BSE_INSTRUMENT (object);
   
-  if (instrument->sample)
-    {
-      bse_object_unref (BSE_OBJECT (instrument->sample));
-      instrument->sample = NULL;
-    }
-  if (instrument->locked_sample)
-    {
-      bse_object_unref (BSE_OBJECT (instrument->locked_sample));
-      instrument->locked_sample = NULL;
-    }
+  g_assert (instrument->type == BSE_INSTRUMENT_NONE); /* paranoid */
   
   /* chain parent class' shutdown handler */
   BSE_OBJECT_CLASS (parent_class)->shutdown (object);
@@ -310,57 +307,101 @@ bse_instrument_do_shutdown (BseObject *object)
 static void
 bse_instrument_unlocked (BseObject *object)
 {
-  BseInstrument *instrument = BSE_INSTRUMENT (object);
+  BseInstrument *instrument;
+
+  instrument = BSE_INSTRUMENT (object);
   
   /* chain parent class' handler */
   if (BSE_OBJECT_CLASS (parent_class)->unlocked)
     BSE_OBJECT_CLASS (parent_class)->unlocked (object);
-  
-  if (instrument->sample != instrument->locked_sample)
+}
+
+static void
+instrument_input_changed (BseInstrument *instrument)
+{
+  if (instrument->type == BSE_INSTRUMENT_SYNTH)
+    bse_object_param_changed (BSE_OBJECT (instrument), "sinstrument");
+  else if (instrument->type == BSE_INSTRUMENT_SAMPLE)
+    bse_object_param_changed (BSE_OBJECT (instrument), "sample");
+  else
+    g_assert_not_reached ();
+}
+
+static void
+unset_input_cb (BseItem *owner,
+		BseItem *input,
+		gpointer data)
+{
+  BseInstrument *instrument = BSE_INSTRUMENT (owner);
+
+  bse_object_remove_notifiers_by_func (input,
+				       instrument_input_changed,
+				       instrument);
+  instrument_input_changed (instrument);
+  instrument->input = NULL;
+  instrument->type = BSE_INSTRUMENT_NONE;
+  if (BSE_IS_SINSTRUMENT (input))
     {
-      if (instrument->locked_sample)
-	bse_object_unref (BSE_OBJECT (instrument->locked_sample));
-      instrument->locked_sample = instrument->sample;
-      if (instrument->locked_sample)
-	bse_object_ref (BSE_OBJECT (instrument->locked_sample));
+      BseSInstrument *sinstrument = BSE_SINSTRUMENT (input);
+
+      bse_sinstrument_poke_foreigns (sinstrument, NULL, sinstrument->voice);
+      if (sinstrument->voice)
+	bse_voice_fade_out (sinstrument->voice);
+    }
+}
+
+static void
+instrument_set_input (BseInstrument  *instrument,
+		      BseSample      *sample,
+		      BseSInstrument *sinstrument)
+{
+  BseItem *item = BSE_ITEM (instrument);
+
+  g_return_if_fail (sample == NULL || sinstrument == NULL);
+  
+  if (instrument->input)
+    bse_item_cross_unref (item, BSE_ITEM (instrument->input));
+
+  if (sample)
+    {
+      instrument->input = BSE_SOURCE (sample);
+      instrument->type = BSE_INSTRUMENT_SAMPLE;
+      bse_item_cross_ref (item, BSE_ITEM (instrument->input), unset_input_cb, NULL);
+      bse_object_add_data_notifier (instrument->input,
+				    "name_set",
+				    instrument_input_changed,
+				    instrument);
+    }
+  else if (sinstrument && sinstrument->instrument == NULL)
+    {
+      instrument->input = BSE_SOURCE (sinstrument);
+      instrument->type = BSE_INSTRUMENT_SYNTH;
+      bse_item_cross_ref (item, BSE_ITEM (instrument->input), unset_input_cb, NULL);
+      bse_object_add_data_notifier (instrument->input,
+				    "name_set",
+				    instrument_input_changed,
+				    instrument);
+      bse_sinstrument_poke_foreigns (sinstrument, instrument, sinstrument->voice);
     }
 }
 
 static void
 bse_instrument_set_param (BseInstrument *instrument,
-			  BseParam	*param)
+			  BseParam	*param,
+			  guint          param_id)
 {
-  BseItem *item = BSE_ITEM (instrument);
   BseEnvelope *env = &instrument->env;
   
-  switch (param->pspec->any.param_id)
+  switch (param_id)
     {
       guint total;
     case PARAM_SAMPLE:
-      if (!param->value.v_item ||
-	  bse_super_get_project (BSE_SUPER (param->value.v_item)) == bse_item_get_project (item))
-	{
-	  if (instrument->sample)
-	    {
-	      bse_object_unref (BSE_OBJECT (instrument->sample));
-	      instrument->type = BSE_INSTRUMENT_NONE;
-	    }
-	  instrument->sample = (BseSample*) param->value.v_item;
-	  if (instrument->sample)
-	    {
-	      bse_object_ref (BSE_OBJECT (instrument->sample));
-	      instrument->type = BSE_INSTRUMENT_SAMPLE;
-	    }
-	}
-      if (!BSE_OBJECT_IS_LOCKED (instrument) &&
-	  instrument->sample != instrument->locked_sample)
-	{
-	  if (instrument->locked_sample)
-	    bse_object_unref (BSE_OBJECT (instrument->locked_sample));
-	  instrument->locked_sample = instrument->sample;
-	  if (instrument->locked_sample)
-	    bse_object_ref (BSE_OBJECT (instrument->locked_sample));
-	}
+      instrument_set_input (instrument, (BseSample*) param->value.v_item, NULL);
+      bse_object_param_changed (BSE_OBJECT (instrument), "sinstrument");
+      break;
+    case PARAM_SYNTH:
+      instrument_set_input (instrument, NULL, (BseSInstrument*) param->value.v_item);
+      bse_object_param_changed (BSE_OBJECT (instrument), "sample");
       break;
     case PARAM_INTERPOLATION:
       instrument->interpolation = param->value.v_bool;
@@ -487,30 +528,42 @@ bse_instrument_set_param (BseInstrument *instrument,
       bse_object_param_changed (BSE_OBJECT (instrument), "envelope");
       break;
     default:
-      g_warning ("%s(\"%s\"): invalid attempt to set parameter \"%s\" of type `%s'",
-                 BSE_OBJECT_TYPE_NAME (instrument),
-                 BSE_OBJECT_NAME (instrument),
-		 param->pspec->any.name,
-		 bse_type_name (param->pspec->type));
+      BSE_UNHANDLED_PARAM_ID (instrument, param, param_id);
       break;
     }
 }
 
 static void
 bse_instrument_get_param (BseInstrument *instrument,
-			  BseParam	*param)
+			  BseParam      *param,
+			  guint          param_id)
 {
   BseEnvelope *env = &instrument->env;
   
-  switch (param->pspec->any.param_id)
+  switch (param_id)
     {
       guint total;
     case PARAM_SAMPLE:
       if (param->value.v_item)
 	bse_object_unref (BSE_OBJECT (param->value.v_item));
-      param->value.v_item = (BseItem*) instrument->sample;
+      if (instrument->type == BSE_INSTRUMENT_SAMPLE && instrument->input)
+	{
+	  param->value.v_item = (BseItem*) instrument->input;
+	  bse_object_ref (BSE_OBJECT (param->value.v_item));
+	}
+      else
+	param->value.v_item = NULL;
+      break;
+    case PARAM_SYNTH:
       if (param->value.v_item)
-	bse_object_ref (BSE_OBJECT (param->value.v_item));
+	bse_object_unref (BSE_OBJECT (param->value.v_item));
+      if (instrument->type == BSE_INSTRUMENT_SYNTH && instrument->input)
+	{
+	  param->value.v_item = (BseItem*) instrument->input;
+	  bse_object_ref (BSE_OBJECT (param->value.v_item));
+	}
+      else
+	param->value.v_item = NULL;
       break;
     case PARAM_INTERPOLATION:
       param->value.v_bool = instrument->interpolation;
@@ -580,11 +633,7 @@ bse_instrument_get_param (BseInstrument *instrument,
       param->value.v_uint = ENVELOPE_TIME_TOTAL (instrument->env);
       break;
     default:
-      g_warning ("%s(\"%s\"): invalid attempt to get parameter \"%s\" of type `%s'",
-                 BSE_OBJECT_TYPE_NAME (instrument),
-                 BSE_OBJECT_NAME (instrument),
-		 param->pspec->any.name,
-		 bse_type_name (param->pspec->type));
+      BSE_UNHANDLED_PARAM_ID (instrument, param, param_id);
       break;
     }
 }
@@ -594,11 +643,31 @@ bse_instrument_set_sample (BseInstrument  *instrument,
 			   BseSample	  *sample)
 {
   g_return_if_fail (BSE_IS_INSTRUMENT (instrument));
-  g_return_if_fail (BSE_IS_SAMPLE (sample));
-  g_return_if_fail (bse_super_get_project (BSE_SUPER (sample)) ==
-		    bse_item_get_project (BSE_ITEM (instrument)));
+  if (sample)
+    {
+      g_return_if_fail (BSE_IS_SAMPLE (sample));
+      g_return_if_fail (bse_item_get_project (BSE_ITEM (instrument)) ==
+			bse_item_get_project (BSE_ITEM (sample)));
+    }
   
   bse_object_set (BSE_OBJECT (instrument),
 		  "sample", sample,
 		  NULL);
 }
+
+void
+bse_instrument_set_sinstrument (BseInstrument  *instrument,
+				BseSInstrument *sinstrument)
+{
+  g_return_if_fail (BSE_IS_INSTRUMENT (instrument));
+  if (sinstrument)
+    {
+      g_return_if_fail (BSE_IS_SINSTRUMENT (sinstrument));
+      g_return_if_fail (bse_item_get_project (BSE_ITEM (instrument)) ==
+			bse_item_get_project (BSE_ITEM (sinstrument)));
+    }
+  
+  bse_object_set (BSE_OBJECT (instrument),
+		  "sinstrument", sinstrument,
+		  NULL);
+}
