@@ -32,11 +32,11 @@ static BseParamSpec*    bse_param_spec_alloc            (BseType         type,
 
 /* --- functions --- */
 gint
-bse_param_values_cmp (BseParam *param1,
-                      BseParam *param2)
+bse_param_values_cmp (const BseParam *param1,
+                      const BseParam *param2)
 {
-  BseParamValue *value1;
-  BseParamValue *value2;
+  const BseParamValue *value1;
+  const BseParamValue *value2;
   const gint failed = -2;
 #define CMP_VALS(m, v1, v2)     ((v1)->m < (v2)->m ? -1 : (v1)->m > (v2)->m)
   
@@ -113,7 +113,7 @@ bse_param_values_cmp (BseParam *param1,
     case BSE_TYPE_PARAM_ITEM:
       return CMP_VALS (v_item, value1, value2); /* FIXME: modification time? */
     default:
-      g_warning ("bse_param_values_equal() used with type `%s'",
+      g_warning ("%s: used with type `%s'", G_STRLOC,
                  bse_type_name (param1->pspec->type));
       return failed;
     }
@@ -180,7 +180,7 @@ bse_param_init_i (BseParam     *param,
       param->value.v_item = NULL;
       break;
     default:
-      g_warning ("bse_param_init() used with type `%s'",
+      g_warning ("%s: used with type `%s'", G_STRLOC,
                  bse_type_name (pspec->type));
       break;
     }
@@ -218,7 +218,7 @@ bse_param_reset_value (BseParam *param)
 }
 
 gboolean
-bse_param_defaults (BseParam *param)
+bse_param_defaults (const BseParam *param)
 {
   BseParam dparam = { NULL, };
   gboolean defaults;
@@ -261,8 +261,8 @@ bse_param_free_value (BseParam *param)
 }
 
 void
-bse_param_copy_value (BseParam *param_src,
-                      BseParam *param_dest)
+bse_param_copy_value (const BseParam *param_src,
+                      BseParam       *param_dest)
 {
   BseParamSpec *pspec;
 
@@ -315,7 +315,7 @@ bse_param_move_value (BseParam *param,
       gfloat *p_float;
       gdouble *p_double;
       BseTime *p_time;
-      guint *p_note;
+      gint *p_note;
       guint *p_index_2d;
       gchar **p_string;
       BseDot **p_dots;
@@ -376,7 +376,7 @@ bse_param_move_value (BseParam *param,
       param->value.v_item = NULL;
       break;
     default:
-      g_warning ("bse_param_move_value() used with type `%s'",
+      g_warning ("%s: used with type `%s'", G_STRLOC,
                  bse_type_name (pspec->type));
       break;
     }
@@ -495,7 +495,7 @@ bse_param_validate (BseParam *param)
         }
       break;
     default:
-      g_warning ("bse_param_validate() used with type `%s'",
+      g_warning ("%s: used with type `%s'", G_STRLOC,
                  bse_type_name (param->pspec->type));
       break;
     }
@@ -564,35 +564,32 @@ bse_param_spec_alloc (BseType       type,
                        G_ALLOC_AND_FREE);
   pspec = g_chunk_new0 (BseParamSpec, bse_param_mem_chunks[type - BSE_TYPE_PARAM_FIRST]);
   pspec->type = type;
-  pspec->any.parent_type = 0;
   pspec->any.name = g_strdup (name);
   g_strcanon (pspec->any.name, "-", '-');
   pspec->any.nick = g_strdup (nick ? nick : pspec->any.name);
   pspec->any.blurb = g_strdup (blurb);
-  pspec->any.param_group = 0;
   pspec->any.flags = flags & (BSE_PARAM_MASK |
                               BSE_PARAM_SERVE_MASK |
                               BSE_PARAM_HINT_MASK);
-  pspec->any.param_id = 0;
   
   return pspec;
 }
 
 void
-bse_param_spec_free (BseParamSpec *pspec)
+bse_param_spec_free_fields (BseParamSpec *static_pspec)
 {
   BseType type;
   
-  g_return_if_fail (BSE_IS_PARAM_SPEC (pspec));
-  g_return_if_fail (pspec->type >= BSE_TYPE_PARAM_FIRST && pspec->type <= BSE_TYPE_PARAM_LAST);
+  g_return_if_fail (BSE_IS_PARAM_SPEC (static_pspec));
+  g_return_if_fail (static_pspec->type >= BSE_TYPE_PARAM_FIRST && static_pspec->type <= BSE_TYPE_PARAM_LAST);
   
-  type = pspec->type;
+  type = static_pspec->type;
   
   switch (type)
     {
     case BSE_TYPE_PARAM_BOOL:
-      g_free (pspec->s_bool.true_identifier);
-      g_free (pspec->s_bool.false_identifier);
+      g_free (static_pspec->s_bool.true_identifier);
+      g_free (static_pspec->s_bool.false_identifier);
       break;
     case BSE_TYPE_PARAM_INT:
     case BSE_TYPE_PARAM_UINT:
@@ -604,29 +601,64 @@ bse_param_spec_free (BseParamSpec *pspec)
     case BSE_TYPE_PARAM_ITEM:
       break;
     case BSE_TYPE_PARAM_ENUM:
-      bse_type_class_unref (pspec->s_enum.enum_class);
+      bse_type_class_unref (static_pspec->s_enum.enum_class);
       break;
     case BSE_TYPE_PARAM_FLAGS:
-      bse_type_class_unref (pspec->s_flags.flags_class);
+      bse_type_class_unref (static_pspec->s_flags.flags_class);
       break;
     case BSE_TYPE_PARAM_STRING:
-      g_free (pspec->s_string.default_value);
-      g_free (pspec->s_string.cset_first);
-      g_free (pspec->s_string.cset_nth);
+      g_free (static_pspec->s_string.default_value);
+      g_free (static_pspec->s_string.cset_first);
+      g_free (static_pspec->s_string.cset_nth);
       break;
     case BSE_TYPE_PARAM_DOTS:
-      g_free (pspec->s_dots.default_dots);
+      g_free (static_pspec->s_dots.default_dots);
       break;
     default:
       break;
     }
-  g_free (pspec->any.name);
-  g_free (pspec->any.nick);
-  g_free (pspec->any.blurb);
+  g_free (static_pspec->any.name);
+  g_free (static_pspec->any.nick);
+  g_free (static_pspec->any.blurb);
   
-  memset (pspec, 0, bse_param_spec_sizes[type - BSE_TYPE_PARAM_FIRST].ssize);
-  
-  g_return_if_fail (bse_param_mem_chunks[type - BSE_TYPE_PARAM_FIRST] != NULL); /* paranoid */
+  memset (static_pspec, 0, bse_param_spec_sizes[type - BSE_TYPE_PARAM_FIRST].ssize);
+}
+
+gpointer
+bse_param_spec_renew (BseParamSpec *pspec,
+		      guint         n_prepend_bytes)
+{
+  guint8 *data;
+  BseType type;
+  guint spec_size;
+
+  g_return_val_if_fail (BSE_IS_PARAM_SPEC (pspec), NULL);
+  g_return_val_if_fail (pspec->type >= BSE_TYPE_PARAM_FIRST && pspec->type <= BSE_TYPE_PARAM_LAST, NULL);
+
+  type = pspec->type;
+
+  spec_size = bse_param_spec_sizes[type - BSE_TYPE_PARAM_FIRST].ssize;
+  data = g_malloc0 (n_prepend_bytes + spec_size);
+  memcpy (data + n_prepend_bytes, pspec, spec_size);
+
+  memset (pspec, 0, spec_size);
+
+  g_chunk_free (pspec, bse_param_mem_chunks[type - BSE_TYPE_PARAM_FIRST]);
+
+  return data;
+}
+
+void
+bse_param_spec_free (BseParamSpec *pspec)
+{
+  BseType type;
+
+  g_return_if_fail (BSE_IS_PARAM_SPEC (pspec));
+  g_return_if_fail (pspec->type >= BSE_TYPE_PARAM_FIRST && pspec->type <= BSE_TYPE_PARAM_LAST);
+
+  type = pspec->type;
+
+  bse_param_spec_free_fields (pspec);
   
   g_chunk_free (pspec, bse_param_mem_chunks[type - BSE_TYPE_PARAM_FIRST]);
 }
@@ -675,7 +707,7 @@ bse_param_spec_uint (const gchar *name,
                      const gchar *blurb,
                      guint        minimum,
                      guint        maximum,
-                     guint        stepping_rate,
+                     gint         stepping_rate,
                      guint        default_value,
                      BseParamBits flags)
 {
@@ -789,10 +821,10 @@ BseParamSpec*
 bse_param_spec_note (const gchar *name,
                      const gchar *nick,
                      const gchar *blurb,
-                     guint        minimum,
-                     guint        maximum,
-                     guint        stepping_rate,
-                     guint        default_value,
+                     gint         minimum,
+                     gint         maximum,
+                     gint         stepping_rate,
+                     gint         default_value,
                      gboolean     allow_void,
                      BseParamBits flags)
 {
@@ -1036,7 +1068,7 @@ bse_param_set_time (BseParam *param,
 
 gboolean
 bse_param_set_note (BseParam *param,
-                    guint     v_note)
+                    gint      v_note)
 {
   g_return_val_if_fail (BSE_IS_PARAM (param), FALSE);
   g_return_val_if_fail (BSE_FUNDAMENTAL_TYPE (param->pspec->type) == BSE_TYPE_PARAM_NOTE, FALSE);
@@ -1190,39 +1222,27 @@ static const struct {
   BseType type2;
   void  (*exchange) (BseParam*, BseParam*);
 } exchange_rules[] = {
-  { BSE_TYPE_PARAM_BOOL,        BSE_TYPE_PARAM_BOOL,            param_exch_copy, },
   { BSE_TYPE_PARAM_BOOL,        BSE_TYPE_PARAM_INT,             param_exch_copy, },
   { BSE_TYPE_PARAM_BOOL,        BSE_TYPE_PARAM_UINT,            param_exch_int_uint, },
   { BSE_TYPE_PARAM_BOOL,        BSE_TYPE_PARAM_ENUM,            param_exch_copy, },
   { BSE_TYPE_PARAM_BOOL,        BSE_TYPE_PARAM_FLAGS,           param_exch_int_uint, },
-  { BSE_TYPE_PARAM_INT,         BSE_TYPE_PARAM_INT,             param_exch_copy, },
   { BSE_TYPE_PARAM_INT,         BSE_TYPE_PARAM_UINT,            param_exch_int_uint, },
   { BSE_TYPE_PARAM_INT,         BSE_TYPE_PARAM_ENUM,            param_exch_copy, },
   { BSE_TYPE_PARAM_INT,         BSE_TYPE_PARAM_FLAGS,           param_exch_int_uint, },
   { BSE_TYPE_PARAM_ENUM,        BSE_TYPE_PARAM_UINT,            param_exch_int_uint, },
-  { BSE_TYPE_PARAM_ENUM,        BSE_TYPE_PARAM_ENUM,            param_exch_copy, },
   { BSE_TYPE_PARAM_ENUM,        BSE_TYPE_PARAM_FLAGS,           param_exch_int_uint, },
   { BSE_TYPE_PARAM_FLAGS,       BSE_TYPE_PARAM_UINT,            param_exch_copy, },
-  { BSE_TYPE_PARAM_FLAGS,       BSE_TYPE_PARAM_FLAGS,           param_exch_copy, },
   { BSE_TYPE_PARAM_FLOAT,       BSE_TYPE_PARAM_BOOL,            param_exch_float_int, },
   { BSE_TYPE_PARAM_FLOAT,       BSE_TYPE_PARAM_INT,             param_exch_float_int, },
   { BSE_TYPE_PARAM_FLOAT,       BSE_TYPE_PARAM_UINT,            param_exch_float_uint, },
   { BSE_TYPE_PARAM_FLOAT,       BSE_TYPE_PARAM_ENUM,            param_exch_float_int, },
   { BSE_TYPE_PARAM_FLOAT,       BSE_TYPE_PARAM_FLAGS,           param_exch_float_uint, },
-  { BSE_TYPE_PARAM_FLOAT,       BSE_TYPE_PARAM_FLOAT,           param_exch_copy, },
   { BSE_TYPE_PARAM_FLOAT,       BSE_TYPE_PARAM_DOUBLE,          param_exch_float_double, },
   { BSE_TYPE_PARAM_DOUBLE,      BSE_TYPE_PARAM_BOOL,            param_exch_double_int, },
   { BSE_TYPE_PARAM_DOUBLE,      BSE_TYPE_PARAM_INT,             param_exch_double_int, },
   { BSE_TYPE_PARAM_DOUBLE,      BSE_TYPE_PARAM_UINT,            param_exch_double_uint, },
   { BSE_TYPE_PARAM_DOUBLE,      BSE_TYPE_PARAM_ENUM,            param_exch_double_int, },
   { BSE_TYPE_PARAM_DOUBLE,      BSE_TYPE_PARAM_FLAGS,           param_exch_double_uint, },
-  { BSE_TYPE_PARAM_DOUBLE,      BSE_TYPE_PARAM_DOUBLE,          param_exch_copy, },
-  { BSE_TYPE_PARAM_TIME,        BSE_TYPE_PARAM_TIME,            param_exch_copy, },
-  { BSE_TYPE_PARAM_NOTE,        BSE_TYPE_PARAM_NOTE,            param_exch_copy, },
-  { BSE_TYPE_PARAM_INDEX_2D,    BSE_TYPE_PARAM_INDEX_2D,        param_exch_copy, },
-  { BSE_TYPE_PARAM_STRING,      BSE_TYPE_PARAM_STRING,          param_exch_copy, },
-  { BSE_TYPE_PARAM_DOTS,        BSE_TYPE_PARAM_DOTS,            param_exch_copy, },
-  { BSE_TYPE_PARAM_ITEM,        BSE_TYPE_PARAM_ITEM,            param_exch_copy, },
 };
 static const guint n_exchange_rules = sizeof (exchange_rules) / sizeof (exchange_rules[0]);
 static inline void
@@ -1233,6 +1253,13 @@ static inline void
 {
   guint i;
   
+  if (type1 == type2)
+    {
+      if (must_swap)
+	*must_swap = FALSE;
+      return param_exch_copy;
+    }
+
   for (i = 0; i < n_exchange_rules; i++)
     {
       if (exchange_rules[i].type1 == type1 &&
@@ -1280,14 +1307,18 @@ bse_param_values_exchange (BseParam *param1,
                                           BSE_FUNDAMENTAL_TYPE (param2->pspec->type),
                                           &must_swap);
   if (exchange_value)
-    exchange_value (must_swap ? param2 : param1, must_swap ? param1 : param2);
+    {
+      exchange_value (must_swap ? param2 : param1, must_swap ? param1 : param2);
+      bse_param_validate (param1);
+      bse_param_validate (param2);
+    }
   
   return exchange_value != NULL;
 }
 
 gboolean
-bse_param_value_convert (BseParam *param_src,
-                         BseParam *param_dest)
+bse_param_value_convert (const BseParam *param_src,
+                         BseParam       *param_dest)
 {
   BseParam tmp = { NULL, };
   gboolean success;
