@@ -94,7 +94,7 @@ store_parasite_list (BseObject  *object,
 	    }
 	  break;
 	default:
-	  g_warning (G_GNUC_PRETTY_FUNCTION "(): unknown parasite type `%c' for \"%s\" in \"%s\"",
+	  g_warning (G_STRLOC ": unknown parasite type `%c' for \"%s\" in \"%s\"",
 		     parasite->type,
 		     name,
 		     BSE_OBJECT_NAME (object));
@@ -147,7 +147,7 @@ fetch_parasite (BseObject *object,
 	    quark_parasite_list = g_quark_from_static_string ("BseParasiteList");
 
 	  if (olist)
-	    bse_object_kill_qdata_no_notify (object, quark_parasite_list);
+	    bse_object_steal_qdata (object, quark_parasite_list);
 	  else
 	    bse_object_add_notifier (object,
 				     "store",
