@@ -27,6 +27,13 @@ BSE_DUMMY_TYPE (BsePcmDeviceAlsa);
 #include        <errno.h>
 
 
+#ifndef HAVE_SND_PCM_CHANNEL_FLUSH
+#  define snd_pcm_playback_drain	snd_pcm_drain_playback
+#  define snd_pcm_channel_flush		snd_pcm_flush_channel
+#  define snd_card_get_longname(y,l)	((*(l)) = NULL)
+#  define snd_card_get_name(y,l)	((*(l)) = NULL)
+#endif
+
 /* --- BsePcmDeviceAlsa structs --- */
 struct _BsePcmDeviceAlsa
 {
@@ -48,7 +55,7 @@ struct _BsePcmDeviceAlsaClass
 static void         bse_pcm_device_alsa_class_init       (BsePcmDeviceAlsaClass   *class);
 static void         bse_pcm_device_alsa_init             (BsePcmDeviceAlsa        *pcm_device_alsa);
 static void         bse_pcm_device_alsa_shutdown         (BseObject              *object);
-static gchar*       bse_pcm_device_alsa_device_name      (BsePcmDevice           *pdev,
+static gchar*       bse_pcm_device_alsa_device_name      (BseDevice              *dev,
 							  gboolean                descriptive);
 static BseErrorType bse_pcm_device_alsa_update_caps      (BsePcmDevice           *pdev);
 static BseErrorType bse_pcm_device_alsa_open             (BsePcmDevice           *pdev,
@@ -65,17 +72,17 @@ static BseErrorType bse_pcm_device_alsa_setup            (BsePcmDeviceAlsa
                                                           guint                   n_fragments,
                                                           guint                   fragment_size);
 static void         bse_pcm_device_alsa_update_state     (BsePcmDevice           *pdev);
-static gboolean     bse_pcm_device_alsa_in_playback      (BsePcmDevice           *pdev);
-static void         bse_pcm_device_alsa_close            (BsePcmDevice           *pdev);
-static guint        bse_pcm_device_alsa_read		 (BsePcmDevice		 *pdev,
+static void	    bse_pcm_device_alsa_retrigger        (BsePcmDevice           *pdev);
+static void         bse_pcm_device_alsa_close            (BseDevice              *pdev);
+static guint        bse_pcm_device_alsa_read		 (BseDevice		 *dev,
 							  guint                   n_bytes,
 							  guint8                 *bytes);
-static guint	    bse_pcm_device_alsa_write		 (BsePcmDevice		 *pdev,
+static guint	    bse_pcm_device_alsa_write		 (BseDevice		 *dev,
 							  guint                   n_bytes,
 							  guint8                  *bytes);
 static guint        pcm_rates_from_alsa_rates            (guint                   alsa_rates);
 
-const char* snd_card_type_name (unsigned int card_type); // FIXME
+static gchar*       pcm_alsa_dup_name                    (unsigned int            card_type);
 
 
 /* --- variables --- */
@@ -114,22 +121,25 @@ static void
 bse_pcm_device_alsa_class_init (BsePcmDeviceAlsaClass *class)
 {
   BseObjectClass *object_class;
+  BseDeviceClass *device_class;
   BsePcmDeviceClass *pcm_device_class;
   
   parent_class = bse_type_class_peek (BSE_TYPE_PCM_DEVICE);
   object_class = BSE_OBJECT_CLASS (class);
+  device_class = BSE_DEVICE_CLASS (class);
   pcm_device_class = BSE_PCM_DEVICE_CLASS (class);
   
   object_class->shutdown = bse_pcm_device_alsa_shutdown;
   
-  pcm_device_class->device_name = bse_pcm_device_alsa_device_name;
+  device_class->close = bse_pcm_device_alsa_close;
+  device_class->device_name = bse_pcm_device_alsa_device_name;
+  device_class->read = bse_pcm_device_alsa_read;
+  device_class->write = bse_pcm_device_alsa_write;
+
   pcm_device_class->update_caps = bse_pcm_device_alsa_update_caps;
   pcm_device_class->open = bse_pcm_device_alsa_open;
   pcm_device_class->update_state = bse_pcm_device_alsa_update_state;
-  pcm_device_class->read = bse_pcm_device_alsa_read;
-  pcm_device_class->write = bse_pcm_device_alsa_write;
-  pcm_device_class->in_playback = bse_pcm_device_alsa_in_playback;
-  pcm_device_class->close = bse_pcm_device_alsa_close;
+  pcm_device_class->retrigger = bse_pcm_device_alsa_retrigger;
 }
 
 static void
@@ -157,10 +167,10 @@ bse_pcm_device_alsa_shutdown (BseObject *object)
 }
 
 static gchar*
-bse_pcm_device_alsa_device_name (BsePcmDevice *pdev,
-				 gboolean      descriptive)
+bse_pcm_device_alsa_device_name (BseDevice *dev,
+				 gboolean   descriptive)
 {
-  BsePcmDeviceAlsa *alsa = BSE_PCM_DEVICE_ALSA (pdev);
+  BsePcmDeviceAlsa *alsa = BSE_PCM_DEVICE_ALSA (dev);
 
   return descriptive ? alsa->card_id : alsa->card_name;
 }
@@ -174,9 +184,10 @@ bse_pcm_device_alsa_open (BsePcmDevice  *pdev,
                           guint          fragment_size)
 {
   BsePcmDeviceAlsa *alsa = BSE_PCM_DEVICE_ALSA (pdev);
+  BseDevice *dev = BSE_DEVICE (alsa);
   BseErrorType error;
   gint omode = 0;
-  gint perrno, fd;
+  gint perrno;
   
   if (readable && writable)
     omode = SND_PCM_OPEN_DUPLEX;
@@ -188,12 +199,13 @@ bse_pcm_device_alsa_open (BsePcmDevice  *pdev,
   perrno = snd_pcm_open (&alsa->handle, alsa->card, alsa->device, omode);
   if (perrno >= 0)
     {
-      fd = snd_pcm_file_descriptor (alsa->handle);
-      if (fd < 0)
-        {
-          snd_pcm_close (alsa->handle);
-          perrno = fd;
-        }
+      /* fd = snd_pcm_file_descriptor (alsa->handle);
+       * if (fd < 0)
+       * {
+       * snd_pcm_close (alsa->handle);
+       * perrno = fd;
+       * }
+       */
     }
   if (perrno < 0)
     {
@@ -206,7 +218,7 @@ bse_pcm_device_alsa_open (BsePcmDevice  *pdev,
       else
         return BSE_ERROR_DEVICE_IO;
     }
-  pdev->pfd.fd = fd;
+  dev->pfd.fd = -1;
   
   error = bse_pcm_device_alsa_setup (alsa, readable, writable, n_channels, rate, 128, fragment_size);
   if (error)
@@ -217,22 +229,25 @@ bse_pcm_device_alsa_open (BsePcmDevice  *pdev,
   else
     {
       if (readable)
-        BSE_OBJECT_SET_FLAGS (alsa, BSE_PCM_FLAG_READABLE);
+        BSE_OBJECT_SET_FLAGS (alsa, BSE_DEVICE_FLAG_READABLE);
       if (writable)
-        BSE_OBJECT_SET_FLAGS (alsa, BSE_PCM_FLAG_WRITABLE);
+        BSE_OBJECT_SET_FLAGS (alsa, BSE_DEVICE_FLAG_WRITABLE);
     }
   
   return error;
 }
 
 static void
-bse_pcm_device_alsa_close (BsePcmDevice *pdev)
+bse_pcm_device_alsa_close (BseDevice *dev)
 {
-  BsePcmDeviceAlsa *alsa = BSE_PCM_DEVICE_ALSA (pdev);
+  BsePcmDeviceAlsa *alsa = BSE_PCM_DEVICE_ALSA (dev);
   
-  snd_pcm_drain_playback (alsa->handle);
+  snd_pcm_playback_drain (alsa->handle);
   snd_pcm_close (alsa->handle);
   alsa->handle = NULL;
+
+  /* chain parent class' handler */
+  BSE_DEVICE_CLASS (parent_class)->close (dev);
 }
 
 static void
@@ -250,7 +265,7 @@ bse_pcm_device_alsa_update_state (BsePcmDevice *pdev)
       channel_status.channel = channel_setup.channel;
       if (snd_pcm_channel_status (alsa->handle, &channel_status) >= 0)
         {
-          pdev->n_playback_bytes = channel_status.count;
+          pdev->n_playback_bytes = channel_status.free;
 	  pdev->playback_buffer_size = channel_setup.buf.block.frags * channel_setup.buf.block.frag_size;
           BSE_IF_DEBUG (PCM)
             g_message ("OSPACE(%s:%d#%d): left=%5d/%d frags: total=%d max=%d min=%d size=%d cur=%d bytes=%d free=%d o=%d u=%d\n",
@@ -302,28 +317,31 @@ bse_pcm_device_alsa_update_state (BsePcmDevice *pdev)
     }
 }
 
-static gboolean
-bse_pcm_device_alsa_in_playback (BsePcmDevice *pdev)
+static void
+bse_pcm_device_alsa_retrigger (BsePcmDevice *pdev)
 {
   BsePcmDeviceAlsa *alsa = BSE_PCM_DEVICE_ALSA (pdev);
-  snd_pcm_channel_setup_t channel_setup = { 0, };
-  snd_pcm_channel_status_t channel_status = { 0, };
 
-  channel_setup.channel = SND_PCM_CHANNEL_PLAYBACK;
-  channel_status.channel = SND_PCM_CHANNEL_PLAYBACK;
-  if (snd_pcm_channel_setup (alsa->handle, &channel_setup) >= 0 &&
-      snd_pcm_channel_status (alsa->handle, &channel_status) >= 0)
-    return channel_status.frag < channel_setup.buf.block.frags;
+  if (0)
+    {
+      if (BSE_DEVICE_READABLE (alsa))
+	snd_pcm_channel_prepare (alsa->handle, SND_PCM_CHANNEL_CAPTURE);
+      if (BSE_DEVICE_WRITABLE (alsa))
+	snd_pcm_channel_prepare (alsa->handle, SND_PCM_CHANNEL_PLAYBACK);
+    }
 
-  return FALSE;
+  if (BSE_DEVICE_READABLE (alsa))
+    snd_pcm_channel_go (alsa->handle, SND_PCM_CHANNEL_CAPTURE);
+  if (BSE_DEVICE_WRITABLE (alsa))
+    snd_pcm_channel_go (alsa->handle, SND_PCM_CHANNEL_PLAYBACK);
 }
 
 static guint
-bse_pcm_device_alsa_read (BsePcmDevice *pdev,
-			  guint         n_bytes,
-			  guint8       *bytes)
+bse_pcm_device_alsa_read (BseDevice *dev,
+			  guint      n_bytes,
+			  guint8    *bytes)
 {
-  BsePcmDeviceAlsa *alsa = BSE_PCM_DEVICE_ALSA (pdev);
+  BsePcmDeviceAlsa *alsa = BSE_PCM_DEVICE_ALSA (dev);
   gint n;
 
   do
@@ -334,11 +352,11 @@ bse_pcm_device_alsa_read (BsePcmDevice *pdev,
 }
 
 static guint
-bse_pcm_device_alsa_write (BsePcmDevice *pdev,
-			   guint         n_bytes,
-			   guint8       *bytes)
+bse_pcm_device_alsa_write (BseDevice *dev,
+			   guint      n_bytes,
+			   guint8    *bytes)
 {
-  BsePcmDeviceAlsa *alsa = BSE_PCM_DEVICE_ALSA (pdev);
+  BsePcmDeviceAlsa *alsa = BSE_PCM_DEVICE_ALSA (dev);
   gint n;
 
   do
@@ -361,13 +379,13 @@ bse_pcm_device_alsa_setup (BsePcmDeviceAlsa *alsa,
   snd_pcm_info_t pcm_info = { 0, };
   guint rep;
 
-  if (snd_pcm_nonblock_mode (alsa->handle, FALSE) < 0)
+  if (snd_pcm_nonblock_mode (alsa->handle, TRUE) < 0)
     return BSE_ERROR_DEVICE_ASYNC;
 
   if (snd_pcm_info (alsa->handle, &pcm_info) < 0)
     return BSE_ERROR_DEVICE_GET_CAPS;
   g_free (alsa->card_name);
-  alsa->card_name = g_strdup (snd_card_type_name (pcm_info.type));
+  alsa->card_name = pcm_alsa_dup_name (pcm_info.type);
   g_free (alsa->card_id);
   alsa->card_id = g_strdup (pcm_info.id);
 
@@ -389,7 +407,7 @@ bse_pcm_device_alsa_setup (BsePcmDeviceAlsa *alsa,
       if (!(channel_info.formats & SND_PCM_FMT_S16))
         return BSE_ERROR_DEVICE_CAPS_MISMATCH;
 
-      if (snd_pcm_flush_channel (alsa->handle, channel_info.channel) < 0)
+      if (snd_pcm_channel_flush (alsa->handle, channel_info.channel) < 0)
         return BSE_ERROR_DEVICE_IO;
 
       channel_params.mode = SND_PCM_MODE_BLOCK;
@@ -401,8 +419,8 @@ bse_pcm_device_alsa_setup (BsePcmDeviceAlsa *alsa,
       channel_params.start_mode = SND_PCM_START_DATA;
       channel_params.stop_mode = SND_PCM_STOP_ROLLOVER;
       channel_params.buf.block.frag_size = fragment_size;
-      channel_params.buf.block.frags_min = 2;
-      channel_params.buf.block.frags_max = -1;
+      channel_params.buf.block.frags_min = 0;
+      channel_params.buf.block.frags_max = 65536;
       if (snd_pcm_channel_params (alsa->handle, &channel_params) < 0)
         return BSE_ERROR_DEVICE_SET_CAPS;
 
@@ -426,7 +444,7 @@ bse_pcm_device_alsa_update_caps (BsePcmDevice *pdev)
   BsePcmDeviceAlsa *alsa = BSE_PCM_DEVICE_ALSA (pdev);
   snd_pcm_info_t pcm_info = { 0, };
   snd_pcm_t *handle = NULL;
-  gint perrno, fd;
+  gint perrno;
   guint rep;
 
   perrno = snd_pcm_open (&handle, alsa->card, alsa->device, SND_PCM_OPEN_DUPLEX);
@@ -436,12 +454,13 @@ bse_pcm_device_alsa_update_caps (BsePcmDevice *pdev)
     perrno = snd_pcm_open (&handle, alsa->card, alsa->device, SND_PCM_OPEN_PLAYBACK);
   if (perrno >= 0)
     {
-      fd = snd_pcm_file_descriptor (handle);
-      if (fd < 0)
-        {
-          snd_pcm_close (handle);
-          perrno = fd;
-        }
+      /* fd = snd_pcm_file_descriptor (handle);
+       * if (fd < 0)
+       * {
+       * snd_pcm_close (handle);
+       * perrno = fd;
+       * }
+       */
     }
   if (perrno < 0)
     {
@@ -457,7 +476,7 @@ bse_pcm_device_alsa_update_caps (BsePcmDevice *pdev)
   if (snd_pcm_info (handle, &pcm_info) < 0)
     return BSE_ERROR_DEVICE_GET_CAPS;
   g_free (alsa->card_name);
-  alsa->card_name = g_strdup (snd_card_type_name (pcm_info.type));
+  alsa->card_name = pcm_alsa_dup_name (pcm_info.type);
   g_free (alsa->card_id);
   alsa->card_id = g_strdup (pcm_info.id);
 
@@ -542,57 +561,74 @@ pcm_rates_from_alsa_rates (guint alsa_rates)
 }
 
 #include <linux/asoundid.h>
-const char* snd_card_type_name (unsigned int card_type)
+#include <malloc.h>
+gchar*
+pcm_alsa_dup_name (unsigned int card_type)
 {
+  gchar *name;
+  char *longname = NULL;
+
+  snd_card_get_longname (card_type, &longname);
+
+  name = g_strdup (longname);
+  free (longname);
+
+  if (name)
+    return name;
+			 
   switch (card_type)
     {
       /* Gravis UltraSound */
-    case SND_CARD_TYPE_GUS_CLASSIC:             return "GUS Classic";
-    case SND_CARD_TYPE_GUS_EXTREME:             return "GUS Extreme";
-    case SND_CARD_TYPE_GUS_ACE:                 return "GUS ACE";
-    case SND_CARD_TYPE_GUS_MAX:                 return "GUS MAX";
-    case SND_CARD_TYPE_AMD_INTERWAVE:           return "AMD Interwave";
+    case SND_CARD_TYPE_GUS_CLASSIC:             name = "GUS Classic";					break;
+    case SND_CARD_TYPE_GUS_EXTREME:             name = "GUS Extreme";					break;
+    case SND_CARD_TYPE_GUS_ACE:                 name = "GUS ACE";					break;
+    case SND_CARD_TYPE_GUS_MAX:                 name = "GUS MAX";					break;
+    case SND_CARD_TYPE_AMD_INTERWAVE:           name = "AMD Interwave";					break;
       /* Sound Blaster */
-    case SND_CARD_TYPE_SB_10:                   return "Sound Blaster 10";
-    case SND_CARD_TYPE_SB_20:                   return "Sound Blaster 20";
-    case SND_CARD_TYPE_SB_PRO:                  return "Sound Blaster Pro";
-    case SND_CARD_TYPE_SB_16:                   return "Sound Blaster 16";
-    case SND_CARD_TYPE_SB_AWE:                  return "Sound Blaster AWE";
+    case SND_CARD_TYPE_SB_10:                   name = "Sound Blaster 10";				break;
+    case SND_CARD_TYPE_SB_20:                   name = "Sound Blaster 20";				break;
+    case SND_CARD_TYPE_SB_PRO:                  name = "Sound Blaster Pro";				break;
+    case SND_CARD_TYPE_SB_16:                   name = "Sound Blaster 16";				break;
+    case SND_CARD_TYPE_SB_AWE:                  name = "Sound Blaster AWE";				break;
       /* Various */
-    case SND_CARD_TYPE_ESS_ES1688:              return "ESS AudioDrive ESx688";
-    case SND_CARD_TYPE_OPL3_SA:                 return "Yamaha OPL3 SA";
-    case SND_CARD_TYPE_MOZART:                  return "OAK Mozart";
-    case SND_CARD_TYPE_S3_SONICVIBES:           return "S3 SonicVibes";
-    case SND_CARD_TYPE_ENS1370:                 return "Ensoniq ES1370";
-    case SND_CARD_TYPE_ENS1371:                 return "Ensoniq ES1371";
-    case SND_CARD_TYPE_CS4232:                  return "CS4232/CS4232A";
-    case SND_CARD_TYPE_CS4236:                  return "CS4235/CS4236B/CS4237B/CS4238B/CS4239";
-    case SND_CARD_TYPE_AMD_INTERWAVE_STB:       return "AMD InterWave + TEA6330T";
-    case SND_CARD_TYPE_ESS_ES1938:              return "ESS Solo-1 ES1938";
-    case SND_CARD_TYPE_ESS_ES18XX:              return "ESS AudioDrive ES18XX";
-    case SND_CARD_TYPE_CS4231:                  return "CS4231";
-    case SND_CARD_TYPE_SERIAL:                  return "Serial MIDI driver";
-    case SND_CARD_TYPE_AD1848:                  return "Generic AD1848 driver";
-    case SND_CARD_TYPE_TRID4DWAVEDX:            return "Trident 4DWave DX";
-    case SND_CARD_TYPE_TRID4DWAVENX:            return "Trident 4DWave NX";
-    case SND_CARD_TYPE_SGALAXY:                 return "Aztech Sound Galaxy";
-    case SND_CARD_TYPE_CS461X:                  return "Sound Fusion CS4610/12/15";
+    case SND_CARD_TYPE_ESS_ES1688:              name = "ESS AudioDrive ESx688";				break;
+    case SND_CARD_TYPE_OPL3_SA:                 name = "Yamaha OPL3 SA";				break;
+    case SND_CARD_TYPE_MOZART:                  name = "OAK Mozart";					break;
+    case SND_CARD_TYPE_S3_SONICVIBES:           name = "S3 SonicVibes";					break;
+    case SND_CARD_TYPE_ENS1370:                 name = "Ensoniq ES1370";				break;
+    case SND_CARD_TYPE_ENS1371:                 name = "Ensoniq ES1371";				break;
+    case SND_CARD_TYPE_CS4232:                  name = "CS4232/CS4232A";				break;
+    case SND_CARD_TYPE_CS4236:                  name = "CS4235/CS4236B/CS4237B/CS4238B/CS4239";		break;
+    case SND_CARD_TYPE_AMD_INTERWAVE_STB:       name = "AMD InterWave + TEA6330T";			break;
+    case SND_CARD_TYPE_ESS_ES1938:              name = "ESS Solo-1 ES1938";				break;
+    case SND_CARD_TYPE_ESS_ES18XX:              name = "ESS AudioDrive ES18XX";				break;
+    case SND_CARD_TYPE_CS4231:                  name = "CS4231";					break;
+    case SND_CARD_TYPE_SERIAL:                  name = "Serial MIDI driver";				break;
+    case SND_CARD_TYPE_AD1848:                  name = "Generic AD1848 driver";				break;
+    case SND_CARD_TYPE_TRID4DWAVEDX:            name = "Trident 4DWave DX";				break;
+    case SND_CARD_TYPE_TRID4DWAVENX:            name = "Trident 4DWave NX";				break;
+    case SND_CARD_TYPE_SGALAXY:                 name = "Aztech Sound Galaxy";				break;
+    case SND_CARD_TYPE_CS461X:                  name = "Sound Fusion CS4610/12/15";			break;
       /* Turtle Beach WaveFront series */
-    case SND_CARD_TYPE_WAVEFRONT:               return "TB WaveFront generic";
-    case SND_CARD_TYPE_TROPEZ:                  return "TB Tropez";
-    case SND_CARD_TYPE_TROPEZPLUS:              return "TB Tropez+";
-    case SND_CARD_TYPE_MAUI:                    return "TB Maui";
-    case SND_CARD_TYPE_CMI8330:                 return "C-Media CMI8330";
-    case SND_CARD_TYPE_DUMMY:                   return "Dummy Soundcard";
+    case SND_CARD_TYPE_WAVEFRONT:               name = "TB WaveFront generic";				break;
+    case SND_CARD_TYPE_TROPEZ:                  name = "TB Tropez";					break;
+    case SND_CARD_TYPE_TROPEZPLUS:              name = "TB Tropez+";					break;
+    case SND_CARD_TYPE_MAUI:                    name = "TB Maui";					break;
+    case SND_CARD_TYPE_CMI8330:                 name = "C-Media CMI8330";				break;
+    case SND_CARD_TYPE_DUMMY:                   name = "Dummy Soundcard";				break;
       /* --- */
-    case SND_CARD_TYPE_ALS100:                  return "Avance Logic ALS100";
+    case SND_CARD_TYPE_ALS100:                  name = "Avance Logic ALS100";				break;
       /* --- */
-    case SND_CARD_TYPE_SHARE:                   return "Soundcard share module";
+    case SND_CARD_TYPE_SHARE:                   name = "Soundcard share module";			break;
     default:
       if (card_type < SND_CARD_TYPE_LAST)
-	return "Unknown";
-      return "Invalid";
+	name = "Unknown";
+      else
+	name = "Invalid";
+      break;
     }
+
+  return g_strdup (name);
 }
 
 #endif	/* BSE_PCM_DEVICE_CONF_ALSA */
