@@ -79,6 +79,7 @@ bse_procedure_base_init (BseProcedureClass *proc)
 {
   proc->name = NULL;
   proc->blurb = NULL;
+  proc->private_id = 0;
   proc->help = NULL;
   proc->author = NULL;
   proc->copyright = NULL;
@@ -126,6 +127,7 @@ bse_procedure_init (BseProcedureClass        *proc,
 
   proc->name = bse_type_name (BSE_PROCEDURE_TYPE (proc));
   proc->blurb = bse_type_blurb (BSE_PROCEDURE_TYPE (proc));
+  proc->private_id = pspec->private_id;
 
   /* init procedure class from plugin,
    * paranoia check certain class members
@@ -164,8 +166,6 @@ bse_procedure_init (BseProcedureClass        *proc,
 	  g_warning ("procedure \"%s\": input parameter \"%s\" has invalid flags",
 		     proc->name,
 		     in_param_specs[i]->any.name);
-	in_param_specs[i]->any.parent_type = BSE_PROCEDURE_TYPE (proc);
-	in_param_specs[i]->any.param_group = g_quark_from_static_string ("Input Parameters");
       }
     else
       break;
@@ -186,8 +186,6 @@ bse_procedure_init (BseProcedureClass        *proc,
 	  g_warning ("procedure \"%s\": output parameter \"%s\" has invalid flags",
 		     proc->name,
 		     out_param_specs[i]->any.name);
-	out_param_specs[i]->any.parent_type = BSE_PROCEDURE_TYPE (proc);
-	out_param_specs[i]->any.param_group = g_quark_from_static_string ("Output Parameters");
       }
     else
       break;
@@ -278,7 +276,7 @@ bse_procedure_execvl (BseProcedureClass *proc,
       lparam = slist ? slist->data : NULL;
       if (!lparam || lparam->pspec != proc->in_param_specs[i])
 	{
-	  g_warning (G_GNUC_PRETTY_FUNCTION "(): input parameter \"%s\" of procedure \"%s\" is invalid or unspecified",
+	  g_warning (G_STRLOC ": input parameter \"%s\" of procedure \"%s\" is invalid or unspecified",
 		     proc->in_param_specs[i]->any.name,
 		     proc->name);
 	  return BSE_ERROR_INTERNAL;
@@ -292,7 +290,7 @@ bse_procedure_execvl (BseProcedureClass *proc,
       lparam = slist ? slist->data : NULL;
       if (!lparam || lparam->pspec != proc->out_param_specs[i])
 	{
-	  g_warning (G_GNUC_PRETTY_FUNCTION "(): output parameter \"%s\" of procedure \"%s\" is invalid or unspecified",
+	  g_warning (G_STRLOC ": output parameter \"%s\" of procedure \"%s\" is invalid or unspecified",
 		     proc->out_param_specs[i]->any.name,
 		     proc->name);
 	  return BSE_ERROR_INTERNAL;
@@ -372,7 +370,7 @@ bse_procedure_execva_i (BseProcedureClass *proc,
       BSE_PARAM_COLLECT_VALUE (param, var_args, error_msg);
       if (error_msg)
 	{
-	  g_warning (G_GNUC_PRETTY_FUNCTION "(): %s", error_msg);
+	  g_warning (G_STRLOC ": %s", error_msg);
 	  g_free (error_msg);
 	  va_end (var_args);
 
