@@ -41,9 +41,11 @@ static void	    bse_sample_class_init		(BseSampleClass *class);
 static void	    bse_sample_init			(BseSample	*sample);
 static void	    bse_sample_do_shutdown		(BseObject	*object);
 static void	    bse_sample_set_param		(BseSample	*sample,
-							 BseParam	*param);
+							 BseParam	*param,
+							 guint           param_id);
 static void	    bse_sample_get_param		(BseSample	*sample,
-							 BseParam	*param);
+							 BseParam	*param,
+							 guint           param_id);
 static void	    bse_sample_do_store_private		(BseObject	*object,
 							 BseStorage	*storage);
 static GTokenType   bse_sample_do_restore		(BseObject	*object,
@@ -69,7 +71,7 @@ BSE_BUILTIN_TYPE (BseSample)
     NULL /* class_data */,
     
     sizeof (BseSample),
-    16 /* n_preallocs */,
+    BSE_PREALLOC_N_SAMPLES /* n_preallocs */,
     (BseObjectInitFunc) bse_sample_init,
   };
   
@@ -160,9 +162,10 @@ bse_sample_do_shutdown (BseObject *object)
 
 static void
 bse_sample_set_param (BseSample *sample,
-		      BseParam *param)
+		      BseParam  *param,
+		      guint      param_id)
 {
-  switch (param->pspec->any.param_id)
+  switch (param_id)
     {
     case PARAM_N_TRACKS:
       sample->n_tracks = param->value.v_uint;
@@ -171,20 +174,17 @@ bse_sample_set_param (BseSample *sample,
       sample->rec_freq = param->value.v_uint;
       break;
     default:
-      g_warning ("%s(\"%s\"): invalid attempt to set parameter \"%s\" of type `%s'",
-		 BSE_OBJECT_TYPE_NAME (sample),
-		 BSE_OBJECT_NAME (sample),
-		 param->pspec->any.name,
-		 bse_type_name (param->pspec->type));
+      BSE_UNHANDLED_PARAM_ID (sample, param, param_id);
       break;
     }
 }
 
 static void
 bse_sample_get_param (BseSample *sample,
-		      BseParam *param)
+		      BseParam  *param,
+		      guint      param_id)
 {
-  switch (param->pspec->any.param_id)
+  switch (param_id)
     {
     case PARAM_N_TRACKS:
       param->value.v_uint = sample->n_tracks;
@@ -193,11 +193,7 @@ bse_sample_get_param (BseSample *sample,
       param->value.v_uint = sample->rec_freq;
       break;
     default:
-      g_warning ("%s(\"%s\"): invalid attempt to get parameter \"%s\" of type `%s'",
-		 BSE_OBJECT_TYPE_NAME (sample),
-		 BSE_OBJECT_NAME (sample),
-		 param->pspec->any.name,
-		 bse_type_name (param->pspec->type));
+      BSE_UNHANDLED_PARAM_ID (sample, param, param_id);
       break;
     }
 }
@@ -273,7 +269,7 @@ bse_sample_fillup_munks (BseSample *sample)
 static inline void
 bse_sample_set_munk_i (BseSample     *sample,
 		       guint	      munk_index,
-		       guint	      recording_note,
+		       gint	      recording_note,
 		       guint	      loop_begin,
 		       guint	      loop_end,
 		       BseBinData    *bin_data)
@@ -295,7 +291,7 @@ bse_sample_set_munk_i (BseSample     *sample,
 void
 bse_sample_set_munk (BseSample	   *sample,
 		     guint	    munk_index,
-		     guint	    recording_note,
+		     gint	    recording_note,
 		     guint	    loop_begin,
 		     guint	    loop_end,
 		     BseBinData    *bin_data)
@@ -411,7 +407,7 @@ parse_munk (BseSample  *sample,
     }
   else if (token_quark == quark_rec_note)
     {
-      guint rec_note;
+      gint rec_note;
       gchar bbuffer[BSE_BBUFFER_SIZE];
       GTokenType expected_token;
       
@@ -450,7 +446,7 @@ bse_sample_do_restore_private (BseObject  *object,
   GScanner *scanner = storage->scanner;
   GTokenType expected_token = BSE_TOKEN_UNMATCHED;
   gchar bbuffer[BSE_BBUFFER_SIZE];
-  guint note;
+  gint note;
   BseMunk munk = {
     BSE_NOTE_VOID,
     0, 0,
