@@ -165,13 +165,13 @@ bse_source_do_shutdown (BseObject *object)
   source = BSE_SOURCE (object);
 
   if (bse_object_get_qdata (object, quark_deferred_input))
-    g_warning ("bse_source_do_shutdown(): Uhh ohh, source still contains deferred_input data");
+    g_warning (G_STRLOC ": Uhh ohh, source still contains deferred_input data");
 
   bse_source_clear_ochannels (source);
 
   if (BSE_SOURCE_PREPARED (source))
     {
-      g_warning ("bse_source_do_shutdown(): Uhh ohh, source still prepared during shutdown");
+      g_warning (G_STRLOC ": Uhh ohh, source still prepared during shutdown");
       bse_source_reset (source);
     }
 
@@ -184,7 +184,7 @@ bse_source_do_shutdown (BseObject *object)
       BseSourceOChannel *oc = BSE_SOURCE_OCHANNEL (source, i + 1);
 
       if (oc->ring_length)
-	g_warning ("bse_source_do_shutdown(): Eeek, output channel %d isn't reset?",
+	g_warning (G_STRLOC ": Eeek, output channel %d isn't reset?",
 		   i + 1);
       g_free (oc->chunks);
     }
@@ -267,6 +267,7 @@ bse_source_class_add_ochannel (BseSourceClass *source_class,
   source_class->ochannel_defs[index].name = g_strdup (name);
   source_class->ochannel_defs[index].blurb = g_strdup (blurb);
   source_class->ochannel_defs[index].n_tracks = n_tracks;
+  source_class->ochannel_defs[index].min_history = 1;
 
   return index + 1;
 }
@@ -401,8 +402,7 @@ bse_source_calc_history (BseSource *source,
   
   g_return_if_fail (oc->in_calc == FALSE); /* paranoid */
 
-  queue_length = 0;	/* FIXME */
-  queue_length = 1;
+  queue_length = BSE_SOURCE_OCHANNEL_DEF (source, ochannel_id)->min_history;
   for (slist = source->outputs; slist; slist = slist->next)
     {
       BseSource *output = slist->data;
@@ -519,7 +519,7 @@ bse_source_do_reset (BseSource *source)
 
       if (oc->in_calc)
 	{
-	  g_warning ("can't reset ochannel contents for %s \"%s\", in_calc flag not cleared",
+	  g_warning (G_STRLOC ": can't reset ochannel contents for %s \"%s\", in_calc flag not cleared",
 		     BSE_OBJECT_TYPE_NAME (source),
 		     BSE_OBJECT_NAME (source));
 	  continue;
@@ -693,11 +693,6 @@ bse_source_do_remove_input (BseSource *source,
 
   osource->outputs = g_slist_remove (osource->outputs, source);
 
-  /* un-prepare (reset) the source if we are the sole prepared
-   * output. FIXME: this can leave loopbacked prepared islands
-   */
-  if (!osource->outputs && BSE_SOURCE_PREPARED (osource))
-    bse_source_reset (osource);
   BSE_NOTIFY (osource, io_changed, NOTIFY (OBJECT, DATA));
   bse_object_unref (BSE_OBJECT (osource));
 }
@@ -922,23 +917,16 @@ bse_source_do_restore_private (BseObject  *object,
   if (!quark_deferred_input)
     quark_deferred_input = g_quark_from_static_string ("BseSourceDeferredInput");
   dinput = g_new (DeferredInput, 1);
-  dinput->next = bse_object_get_qdata (object, quark_deferred_input);
+  dinput->next = bse_object_steal_qdata (object, quark_deferred_input);
   dinput->ichannel_id = ichannel_id;
   dinput->osource_path = osource_path;
   dinput->ochannel_name = g_strdup (scanner->value.v_string);
-  if (dinput->next)
-    {
-      bse_object_kill_qdata_no_notify (object, quark_deferred_input);
-      bse_object_set_qdata_full (object, quark_deferred_input, dinput, deferred_input_free);
-    }
-  else
-    {
-      bse_object_set_qdata_full (object, quark_deferred_input, dinput, deferred_input_free);
-      bse_object_add_data_notifier (BSE_OBJECT (project),
-				    "complete_restore",
-				    resolve_dinput,
-				    source);
-    }
+  bse_object_set_qdata_full (object, quark_deferred_input, dinput, deferred_input_free);
+  if (!dinput->next)
+    bse_object_add_data_notifier (BSE_OBJECT (project),
+				  "complete_restore",
+				  resolve_dinput,
+				  source);
 
   /* read closing brace */
   return g_scanner_get_next_token (scanner) == ')' ? G_TOKEN_NONE : ')';
