@@ -50,9 +50,11 @@ static void	   bse_gen_osc_class_init       (BseGenOscClass	*class);
 static void	   bse_gen_osc_class_destroy    (BseGenOscClass	*class);
 static void	   bse_gen_osc_do_shutdown      (BseObject     	*object);
 static void        bse_gen_osc_set_param        (BseGenOsc	*gen_osc,
-						 BseParam       *param);
+						 BseParam       *param,
+						 guint           param_id);
 static void        bse_gen_osc_get_param        (BseGenOsc	*gen_osc,
-						 BseParam       *param);
+						 BseParam       *param,
+						 guint           param_id);
 static void        bse_gen_osc_prepare          (BseSource      *source,
 						 BseIndex        index);
 static BseChunk*   bse_gen_osc_calc_chunk       (BseSource      *source,
@@ -186,9 +188,9 @@ bse_gen_osc_class_init (BseGenOscClass *class)
 						    BSE_PARAM_DEFAULT |
 						    BSE_PARAM_HINT_SCALE));
   
-  ochannel_id = bse_source_class_add_ochannel (source_class, "OscOut", "Mono Oscillated Output", 1);
+  ochannel_id = bse_source_class_add_ochannel (source_class, "mono_out", "Mono Oscillated Output", 1);
   g_assert (ochannel_id == BSE_GEN_OSC_OCHANNEL_MONO);
-  ichannel_id = bse_source_class_add_ichannel (source_class, "FreqMod", "Mono Frequency Modulation Input", 1, 1);
+  ichannel_id = bse_source_class_add_ichannel (source_class, "freq_mod_in", "Mono Frequency Modulation Input", 1, 1);
   g_assert (ichannel_id == BSE_GEN_OSC_ICHANNEL_FREQ_MOD);
 }
 
@@ -283,8 +285,7 @@ bse_gen_osc_class_ref_tables (BseGenOscClass *class)
 	  table[i] = 0.5 + (table_size - d) / table_size * BSE_MAX_SAMPLE_VALUE;
 	}
 #if 0
-      if (!(i%(guint)(table_size/100)))
-	g_print ("triangle_table[%06u]=%d\n", i, table[i]);
+      g_print ("triangle_table[%06u]=%d\n", i, table[i]);
 #endif
     }
   class->triangle_table = table;
@@ -393,11 +394,12 @@ bse_gen_osc_update_locals (BseGenOsc *gosc)
 
 static void
 bse_gen_osc_set_param (BseGenOsc *gosc,
-		       BseParam  *param)
+		       BseParam  *param,
+		       guint      param_id)
 {
   guint wave = 0;
 
-  switch (param->pspec->any.param_id)
+  switch (param_id)
     {
     case PARAM_WAVE_FORM:
       gosc->wave = param->value.v_enum;
@@ -463,20 +465,17 @@ bse_gen_osc_set_param (BseGenOsc *gosc,
       bse_gen_osc_update_locals (gosc);
       break;
     default:
-      g_warning ("%s(\"%s\"): invalid attempt to set parameter \"%s\" of type `%s'",
-		 BSE_OBJECT_TYPE_NAME (gosc),
-		 BSE_OBJECT_NAME (gosc),
-		 param->pspec->any.name,
-		 bse_type_name (param->pspec->type));
+      BSE_UNHANDLED_PARAM_ID (gosc, param, param_id);
       break;
     }
 }
 
 static void
 bse_gen_osc_get_param (BseGenOsc *gosc,
-		       BseParam  *param)
+                       BseParam  *param,
+		       guint      param_id)
 {
-  switch (param->pspec->any.param_id)
+  switch (param_id)
     {
     case PARAM_WAVE_FORM:
       param->value.v_enum = gosc->wave;
@@ -515,11 +514,7 @@ bse_gen_osc_get_param (BseGenOsc *gosc,
       param->value.v_float = gosc->self_perc;
       break;
     default:
-      g_warning ("%s(\"%s\"): invalid attempt to get parameter \"%s\" of type `%s'",
-		 BSE_OBJECT_TYPE_NAME (gosc),
-		 BSE_OBJECT_NAME (gosc),
-		 param->pspec->any.name,
-		 bse_type_name (param->pspec->type));
+      BSE_UNHANDLED_PARAM_ID (gosc, param, param_id);
       break;
     }
 }
